@@ -1,4 +1,4 @@
-// Copyright © 2018–2021 Trevor Spiteri
+// Copyright © 2018–2022 Trevor Spiteri
 
 // This library is free software: you can redistribute it and/or
 // modify it under the terms of either
@@ -14,14 +14,12 @@
 // <https://opensource.org/licenses/MIT>.
 
 use crate::{
-    helpers::IntHelper,
+    int256::{self, I256, U256},
     traits::ToFixed,
     types::extra::{LeEqU128, LeEqU16, LeEqU32, LeEqU64, LeEqU8},
-    wide_div::WideDivRem,
     FixedI128, FixedI16, FixedI32, FixedI64, FixedI8, FixedU128, FixedU16, FixedU32, FixedU64,
     FixedU8,
 };
-use az_crate::WrappingAs;
 #[cfg(not(debug_assertions))]
 use core::hint;
 use core::{
@@ -241,7 +239,7 @@ macro_rules! fixed_arith {
             type Output = $Fixed<Frac>;
             #[inline]
             fn mul(self, rhs: $Fixed<Frac>) -> $Fixed<Frac> {
-                let (ans, overflow) = self.to_bits().mul_overflow(rhs.to_bits(), Frac::U32);
+                let (ans, overflow) = overflowing_mul(self.to_bits(), rhs.to_bits(), Frac::U32);
                 debug_assert!(!overflow, "overflow");
                 Self::from_bits(ans)
             }
@@ -252,7 +250,7 @@ macro_rules! fixed_arith {
         impl<Frac, RhsFrac: $LeEqU> MulAssign<$Fixed<RhsFrac>> for $Fixed<Frac> {
             #[inline]
             fn mul_assign(&mut self, rhs: $Fixed<RhsFrac>) {
-                let (ans, overflow) = self.to_bits().mul_overflow(rhs.to_bits(), RhsFrac::U32);
+                let (ans, overflow) = overflowing_mul(self.to_bits(), rhs.to_bits(), RhsFrac::U32);
                 debug_assert!(!overflow, "overflow");
                 *self = Self::from_bits(ans);
             }
@@ -261,7 +259,7 @@ macro_rules! fixed_arith {
         impl<Frac, RhsFrac: $LeEqU> MulAssign<&$Fixed<RhsFrac>> for $Fixed<Frac> {
             #[inline]
             fn mul_assign(&mut self, rhs: &$Fixed<RhsFrac>) {
-                let (ans, overflow) = self.to_bits().mul_overflow(rhs.to_bits(), RhsFrac::U32);
+                let (ans, overflow) = overflowing_mul(self.to_bits(), rhs.to_bits(), RhsFrac::U32);
                 debug_assert!(!overflow, "overflow");
                 *self = Self::from_bits(ans);
             }
@@ -271,7 +269,7 @@ macro_rules! fixed_arith {
             type Output = $Fixed<Frac>;
             #[inline]
             fn div(self, rhs: $Fixed<Frac>) -> $Fixed<Frac> {
-                let (ans, overflow) = self.to_bits().div_overflow(rhs.to_bits(), Frac::U32);
+                let (ans, overflow) = overflowing_div(self.to_bits(), rhs.to_bits(), Frac::U32);
                 debug_assert!(!overflow, "overflow");
                 Self::from_bits(ans)
             }
@@ -588,18 +586,40 @@ fixed_arith! { FixedI32(i32, LeEqU32, 32, NonZeroI32), Signed }
 fixed_arith! { FixedI64(i64, LeEqU64, 64, NonZeroI64), Signed }
 fixed_arith! { FixedI128(i128, LeEqU128, 128, NonZeroI128), Signed }
 
-pub(crate) trait MulDivOverflow: Sized {
-    fn mul_overflow(self, rhs: Self, frac_nbits: u32) -> (Self, bool);
+pub(crate) trait OverflowingMulDiv: Sized {
+    // 0 <= frac_nbits <= NBITS
+    fn overflowing_mul(self, rhs: Self, frac_nbits: u32) -> (Self, bool);
     // -NBITS <= frac_nbits <= 2 * NBITS
-    fn mul_add_overflow(self, mul: Self, add: Self, frac_nbits: i32) -> (Self, bool);
-    fn div_overflow(self, rhs: Self, frac_nbits: u32) -> (Self, bool);
+    fn overflowing_mul_add(self, mul: Self, add: Self, frac_nbits: i32) -> (Self, bool);
+    // 0 <= frac_nbits <= NBITS
+    fn overflowing_div(self, rhs: Self, frac_nbits: u32) -> (Self, bool);
+}
+
+#[inline]
+pub(crate) fn overflowing_mul<O: OverflowingMulDiv>(lhs: O, rhs: O, frac_nbits: u32) -> (O, bool) {
+    lhs.overflowing_mul(rhs, frac_nbits)
+}
+
+#[inline]
+pub(crate) fn overflowing_mul_add<O: OverflowingMulDiv>(
+    mul1: O,
+    mul2: O,
+    add: O,
+    frac_nbits: i32,
+) -> (O, bool) {
+    mul1.overflowing_mul_add(mul2, add, frac_nbits)
+}
+
+#[inline]
+pub(crate) fn overflowing_div<O: OverflowingMulDiv>(lhs: O, rhs: O, frac_nbits: u32) -> (O, bool) {
+    lhs.overflowing_div(rhs, frac_nbits)
 }
 
 macro_rules! mul_div_widen {
-    ($Single:ty, $Double:ty, $Signedness:tt) => {
-        impl MulDivOverflow for $Single {
+    ($Single:ty, $Double:ty, $Signedness:tt, $Unsigned:ty) => {
+        impl OverflowingMulDiv for $Single {
             #[inline]
-            fn mul_overflow(self, rhs: $Single, frac_nbits: u32) -> ($Single, bool) {
+            fn overflowing_mul(self, rhs: $Single, frac_nbits: u32) -> ($Single, bool) {
                 const NBITS: u32 = <$Single>::BITS;
                 let int_nbits: u32 = NBITS - frac_nbits;
                 let lhs2 = <$Double>::from(self);
@@ -609,13 +629,12 @@ macro_rules! mul_div_widen {
             }
 
             #[inline]
-            fn mul_add_overflow(
+            fn overflowing_mul_add(
                 self,
                 mul: $Single,
                 add: $Single,
                 mut frac_nbits: i32,
             ) -> ($Single, bool) {
-                type Unsigned = <$Single as IntHelper>::Unsigned;
                 const NBITS: i32 = <$Single>::BITS as i32;
                 let self2 = <$Double>::from(self);
                 let mul2 = <$Double>::from(mul);
@@ -623,7 +642,7 @@ macro_rules! mul_div_widen {
                 let (prod2, overflow2) = if frac_nbits < 0 {
                     frac_nbits += NBITS;
                     debug_assert!(frac_nbits >= 0);
-                    prod2.overflowing_mul(<$Double>::from(Unsigned::MAX) + 1)
+                    prod2.overflowing_mul(<$Double>::from(<$Unsigned>::MAX) + 1)
                 } else if frac_nbits > NBITS {
                     frac_nbits -= NBITS;
                     debug_assert!(frac_nbits <= NBITS);
@@ -631,12 +650,12 @@ macro_rules! mul_div_widen {
                 } else {
                     (prod2, false)
                 };
-                let lo = (prod2 >> frac_nbits) as Unsigned;
+                let lo = (prod2 >> frac_nbits) as $Unsigned;
                 let hi = (prod2 >> frac_nbits >> NBITS) as $Single;
                 if_signed_unsigned!(
                     $Signedness,
                     {
-                        let (uns, carry) = lo.overflowing_add(add as Unsigned);
+                        let (uns, carry) = lo.overflowing_add(add as $Unsigned);
                         let ans = uns as $Single;
                         let expected_hi = if (ans.is_negative() != add.is_negative()) == carry {
                             0
@@ -653,7 +672,7 @@ macro_rules! mul_div_widen {
             }
 
             #[inline]
-            fn div_overflow(self, rhs: $Single, frac_nbits: u32) -> ($Single, bool) {
+            fn overflowing_div(self, rhs: $Single, frac_nbits: u32) -> ($Single, bool) {
                 const NBITS: u32 = <$Single>::BITS;
                 let lhs2 = <$Double>::from(self) << frac_nbits;
                 let rhs2 = <$Double>::from(rhs);
@@ -670,257 +689,130 @@ macro_rules! mul_div_widen {
     };
 }
 
-trait FallbackHelper: Sized {
-    type Unsigned;
-    fn hi_lo(self) -> (Self, Self);
-    fn shift_lo_up(self) -> Self;
-    fn shift_lo_up_unsigned(self) -> Self::Unsigned;
-    fn combine_lo_then_shl(self, lo: Self::Unsigned, shift: u32) -> (Self, bool);
-    fn combine_lo_then_shl_add(self, lo: Self::Unsigned, shift: u32, add: Self) -> (Self, bool);
-    fn carrying_add(self, other: Self) -> (Self, Self);
-}
+mul_div_widen! { u8, u16, Unsigned, u8 }
+mul_div_widen! { u16, u32, Unsigned, u16 }
+mul_div_widen! { u32, u64, Unsigned, u32 }
+mul_div_widen! { u64, u128, Unsigned, u64 }
+mul_div_widen! { i8, i16, Signed, u8 }
+mul_div_widen! { i16, i32, Signed, u16 }
+mul_div_widen! { i32, i64, Signed, u32 }
+mul_div_widen! { i64, i128, Signed, u64 }
 
-impl FallbackHelper for u128 {
-    type Unsigned = u128;
+impl OverflowingMulDiv for u128 {
     #[inline]
-    fn hi_lo(self) -> (u128, u128) {
-        (self >> 64, self & !(!0 << 64))
-    }
-
-    #[inline]
-    fn shift_lo_up(self) -> u128 {
-        debug_assert!(self >> 64 == 0);
-        self << 64
-    }
-
-    #[inline]
-    fn shift_lo_up_unsigned(self) -> u128 {
-        debug_assert!(self >> 64 == 0);
-        self << 64
-    }
-
-    #[inline]
-    fn combine_lo_then_shl(self, lo: u128, shift: u32) -> (u128, bool) {
-        if shift == 128 {
-            (self, false)
-        } else if shift == 0 {
-            (lo, self != 0)
+    fn overflowing_mul(self, rhs: u128, frac_nbits: u32) -> (u128, bool) {
+        if frac_nbits == 0 {
+            self.overflowing_mul(rhs)
         } else {
-            let lo = lo >> shift;
-            let hi = self << (128 - shift);
-            (lo | hi, self >> shift != 0)
+            let prod = int256::wide_mul_u128(self, rhs);
+            int256::overflowing_shl_u256_into_u128(prod, frac_nbits)
         }
     }
 
     #[inline]
-    fn combine_lo_then_shl_add(self, lo: u128, shift: u32, add: u128) -> (u128, bool) {
-        let (combine, overflow1) = if shift == 128 {
-            (self, false)
-        } else if shift == 0 {
-            (lo, self != 0)
-        } else {
-            let lo = lo >> shift;
-            let hi = self << (128 - shift);
-            (lo | hi, self >> shift != 0)
-        };
-        let (ans, overflow2) = combine.overflowing_add(add);
-        (ans, overflow1 || overflow2)
+    fn overflowing_mul_add(self, mul: u128, add: u128, mut frac_nbits: i32) -> (u128, bool) {
+        // l * r + a
+        let mut prod = int256::wide_mul_u128(self, mul);
+
+        let mut overflow1 = false;
+        if frac_nbits < 0 {
+            frac_nbits += 128;
+            debug_assert!(frac_nbits >= 0);
+            overflow1 = prod.hi != 0;
+            prod.hi = prod.lo;
+            prod.lo = 0;
+        } else if frac_nbits > 128 {
+            frac_nbits -= 128;
+            debug_assert!(frac_nbits <= 128);
+            prod.lo = prod.hi;
+            prod.hi = 0;
+        }
+
+        let (shifted, overflow2) = int256::overflowing_shl_u256_into_u128(prod, frac_nbits as u32);
+        let (ans, overflow3) = shifted.overflowing_add(add);
+        (ans, overflow1 | overflow2 | overflow3)
     }
 
     #[inline]
-    fn carrying_add(self, rhs: u128) -> (u128, u128) {
-        let (sum, overflow) = self.overflowing_add(rhs);
-        let carry = if overflow { 1 } else { 0 };
-        (sum, carry)
+    fn overflowing_div(self, rhs: u128, frac_nbits: u32) -> (u128, bool) {
+        if frac_nbits == 0 {
+            self.overflowing_div(rhs)
+        } else {
+            let lhs2 = U256 {
+                lo: self << frac_nbits,
+                hi: self >> (128 - frac_nbits),
+            };
+            let (quot2, _) = int256::div_rem_u256_u128(lhs2, rhs);
+            let quot = quot2.lo;
+            let overflow = quot2.hi != 0;
+            (quot, overflow)
+        }
     }
 }
 
-impl FallbackHelper for i128 {
-    type Unsigned = u128;
+impl OverflowingMulDiv for i128 {
     #[inline]
-    fn hi_lo(self) -> (i128, i128) {
-        (self >> 64, self & !(!0 << 64))
-    }
-
-    #[inline]
-    fn shift_lo_up(self) -> i128 {
-        debug_assert!(self >> 64 == 0);
-        self << 64
-    }
-
-    #[inline]
-    fn shift_lo_up_unsigned(self) -> u128 {
-        debug_assert!(self >> 64 == 0);
-        (self << 64) as u128
-    }
-
-    #[inline]
-    fn combine_lo_then_shl(self, lo: u128, shift: u32) -> (i128, bool) {
-        if shift == 128 {
-            (self, false)
-        } else if shift == 0 {
-            let ans = lo as i128;
-            (ans, self != if ans < 0 { -1 } else { 0 })
+    fn overflowing_mul(self, rhs: i128, frac_nbits: u32) -> (i128, bool) {
+        if frac_nbits == 0 {
+            self.overflowing_mul(rhs)
         } else {
-            let lo = (lo >> shift) as i128;
-            let hi = self << (128 - shift);
-            let ans = lo | hi;
-            (ans, self >> shift != if ans < 0 { -1 } else { 0 })
+            let prod = int256::wide_mul_i128(self, rhs);
+            int256::overflowing_shl_i256_into_i128(prod, frac_nbits)
         }
     }
 
     #[inline]
-    fn combine_lo_then_shl_add(self, lo: u128, shift: u32, add: i128) -> (i128, bool) {
-        let (combine_lo, combine_hi) = if shift == 128 {
-            (self as u128, if self < 0 { -1 } else { 0 })
-        } else if shift == 0 {
-            (lo, self)
-        } else {
-            (
-                (lo >> shift) | (self << (128 - shift)) as u128,
-                self >> shift,
-            )
-        };
-        let (uns, carry) = combine_lo.overflowing_add(add as u128);
+    fn overflowing_mul_add(self, mul: i128, add: i128, mut frac_nbits: i32) -> (i128, bool) {
+        // l * r + a
+        let mut prod = int256::wide_mul_i128(self, mul);
+
+        let mut overflow1 = false;
+        if frac_nbits < 0 {
+            frac_nbits += 128;
+            debug_assert!(frac_nbits >= 0);
+            overflow1 = prod.hi != (prod.lo as i128) >> 127;
+            prod.hi = prod.lo as i128;
+            prod.lo = 0;
+        } else if frac_nbits > 128 {
+            frac_nbits -= 128;
+            debug_assert!(frac_nbits <= 128);
+            prod.lo = prod.hi as u128;
+            prod.hi >>= 127;
+        }
+
+        let shifted = int256::shl_i256_max_128(prod, frac_nbits as u32);
+        let (uns, carry) = shifted.lo.overflowing_add(add as u128);
         let ans = uns as i128;
-        let mut expected_hi = if ans < 0 { -1 } else { 0 };
+        let mut expected_hi = ans >> 127;
         if add < 0 {
             expected_hi += 1;
         }
         if carry {
             expected_hi -= 1;
         }
-        (ans, combine_hi != expected_hi)
+        (ans, overflow1 | (shifted.hi != expected_hi))
     }
 
     #[inline]
-    fn carrying_add(self, rhs: i128) -> (i128, i128) {
-        let (sum, overflow) = self.overflowing_add(rhs);
-        let carry = if overflow {
-            if sum < 0 {
-                1
-            } else {
-                -1
-            }
+    fn overflowing_div(self, rhs: i128, frac_nbits: u32) -> (i128, bool) {
+        if frac_nbits == 0 {
+            self.overflowing_div(rhs)
         } else {
-            0
-        };
-        (sum, carry)
+            let lhs2 = I256 {
+                lo: (self << frac_nbits) as u128,
+                hi: self >> (128 - frac_nbits),
+            };
+            let (quot2, _) = int256::div_rem_i256_i128(lhs2, rhs);
+            let quot = quot2.lo as i128;
+            let overflow = quot2.hi != quot >> 127;
+            (quot, overflow)
+        }
     }
 }
 
-macro_rules! mul_div_fallback {
-    ($Single:ty, $Uns:ty, $Signedness:tt) => {
-        impl MulDivOverflow for $Single {
-            #[inline]
-            fn mul_overflow(self, rhs: $Single, frac_nbits: u32) -> ($Single, bool) {
-                if frac_nbits == 0 {
-                    self.overflowing_mul(rhs)
-                } else {
-                    let (lh, ll) = self.hi_lo();
-                    let (rh, rl) = rhs.hi_lo();
-                    let ll_rl = ll.wrapping_mul(rl);
-                    let lh_rl = lh.wrapping_mul(rl);
-                    let ll_rh = ll.wrapping_mul(rh);
-                    let lh_rh = lh.wrapping_mul(rh);
-
-                    let col01 = ll_rl as <$Single as FallbackHelper>::Unsigned;
-                    let (col01_hi, col01_lo) = col01.hi_lo();
-                    let partial_col12 = lh_rl + col01_hi as $Single;
-                    let (col12, carry_col3) = partial_col12.carrying_add(ll_rh);
-                    let (col12_hi, col12_lo) = col12.hi_lo();
-                    let (_, carry_col3_lo) = carry_col3.hi_lo();
-                    let ans01 = col12_lo.shift_lo_up_unsigned() + col01_lo;
-                    let ans23 = lh_rh + col12_hi + carry_col3_lo.shift_lo_up();
-                    ans23.combine_lo_then_shl(ans01, frac_nbits)
-                }
-            }
-
-            #[inline]
-            fn mul_add_overflow(
-                self,
-                mul: $Single,
-                add: $Single,
-                mut frac_nbits: i32,
-            ) -> ($Single, bool) {
-                const NBITS: i32 = <$Single>::BITS as i32;
-
-                // l * r + a
-                let (lh, ll) = self.hi_lo();
-                let (rh, rl) = mul.hi_lo();
-                let ll_rl = ll.wrapping_mul(rl);
-                let lh_rl = lh.wrapping_mul(rl);
-                let ll_rh = ll.wrapping_mul(rh);
-                let lh_rh = lh.wrapping_mul(rh);
-
-                let col01 = ll_rl as <$Single as FallbackHelper>::Unsigned;
-                let (col01_hi, col01_lo) = col01.hi_lo();
-                let partial_col12 = lh_rl + col01_hi as $Single;
-                let (col12, carry_col3) = partial_col12.carrying_add(ll_rh);
-                let (col12_hi, col12_lo) = col12.hi_lo();
-                let (_, carry_col3_lo) = carry_col3.hi_lo();
-                let mut ans01 = col12_lo.shift_lo_up_unsigned() + col01_lo;
-                let mut ans23 = lh_rh + col12_hi + carry_col3_lo.shift_lo_up();
-
-                let mut overflow2 = false;
-                if frac_nbits < 0 {
-                    frac_nbits += NBITS;
-                    debug_assert!(frac_nbits >= 0);
-                    let expected_ans23 = if_signed_unsigned!(
-                        $Signedness,
-                        ans01.wrapping_as::<$Single>() >> (NBITS - 1),
-                        0,
-                    );
-                    overflow2 = ans23 != expected_ans23;
-                    ans23 = ans01.wrapping_as::<$Single>();
-                    ans01 = 0;
-                } else if frac_nbits > NBITS {
-                    frac_nbits -= NBITS;
-                    debug_assert!(frac_nbits <= NBITS);
-                    let sign_extension = if_signed_unsigned!($Signedness, ans23 >> (NBITS - 1), 0);
-                    ans01 = ans23.wrapping_as::<$Uns>();
-                    ans23 = sign_extension;
-                }
-
-                let (ans, overflow) = ans23.combine_lo_then_shl_add(ans01, frac_nbits as u32, add);
-                (ans, overflow2 || overflow)
-            }
-
-            #[inline]
-            fn div_overflow(self, rhs: $Single, frac_nbits: u32) -> ($Single, bool) {
-                if frac_nbits == 0 {
-                    self.overflowing_div(rhs)
-                } else {
-                    const NBITS: u32 = <$Single>::BITS;
-                    let lhs2 = (self >> (NBITS - frac_nbits), (self << frac_nbits) as $Uns);
-                    let (quot2, _) = rhs.div_rem_from(lhs2);
-                    let quot = quot2.1 as $Single;
-                    let overflow = if_signed_unsigned!(
-                        $Signedness,
-                        quot2.0 != if quot < 0 { -1 } else { 0 },
-                        quot2.0 != 0
-                    );
-                    (quot, overflow)
-                }
-            }
-        }
-    };
-}
-
-mul_div_widen! { u8, u16, Unsigned }
-mul_div_widen! { u16, u32, Unsigned }
-mul_div_widen! { u32, u64, Unsigned }
-mul_div_widen! { u64, u128, Unsigned }
-mul_div_fallback! { u128, u128, Unsigned }
-mul_div_widen! { i8, i16, Signed }
-mul_div_widen! { i16, i32, Signed }
-mul_div_widen! { i32, i64, Signed }
-mul_div_widen! { i64, i128, Signed }
-mul_div_fallback! { i128, u128, Signed }
-
 #[cfg(test)]
 mod tests {
-    use crate::{types::extra::Unsigned, *};
+    use crate::{arith, types::extra::Unsigned, *};
 
     #[test]
     fn fixed_u16() {
@@ -1333,38 +1225,62 @@ mod tests {
     }
 
     #[test]
-    fn mul_add_overflow_large_frac_nbits() {
+    fn overflowing_mul_add_large_frac_nbits() {
         let nbits_2 = 128;
 
         let max = u64::MAX;
 
-        assert_eq!(max.mul_add_overflow(max, max, nbits_2), (max, false));
-        assert_eq!(max.mul_add_overflow(max, max, nbits_2 - 1), (0, true));
         assert_eq!(
-            max.mul_add_overflow(max, max - 1, nbits_2 - 1),
+            arith::overflowing_mul_add(max, max, max, nbits_2),
+            (max, false)
+        );
+        assert_eq!(
+            arith::overflowing_mul_add(max, max, max, nbits_2 - 1),
+            (0, true)
+        );
+        assert_eq!(
+            arith::overflowing_mul_add(max, max, max - 1, nbits_2 - 1),
             (max, false)
         );
 
         let (min, max) = (i64::MIN, i64::MAX);
 
-        assert_eq!(max.mul_add_overflow(max, max, nbits_2 - 2), (max, false));
-        assert_eq!(max.mul_add_overflow(max, max, nbits_2 - 3), (min, true));
         assert_eq!(
-            max.mul_add_overflow(max, max - 1, nbits_2 - 3),
+            arith::overflowing_mul_add(max, max, max, nbits_2 - 2),
+            (max, false)
+        );
+        assert_eq!(
+            arith::overflowing_mul_add(max, max, max, nbits_2 - 3),
+            (min, true)
+        );
+        assert_eq!(
+            arith::overflowing_mul_add(max, max, max - 1, nbits_2 - 3),
             (max, false)
         );
 
-        assert_eq!(min.mul_add_overflow(min, max, nbits_2 - 1), (max, false));
-        assert_eq!(min.mul_add_overflow(min, max, nbits_2 - 2), (min, true));
         assert_eq!(
-            min.mul_add_overflow(min, max - 1, nbits_2 - 2),
+            arith::overflowing_mul_add(min, min, max, nbits_2 - 1),
+            (max, false)
+        );
+        assert_eq!(
+            arith::overflowing_mul_add(min, min, max, nbits_2 - 2),
+            (min, true)
+        );
+        assert_eq!(
+            arith::overflowing_mul_add(min, min, max - 1, nbits_2 - 2),
             (max, false)
         );
 
-        assert_eq!(max.mul_add_overflow(min, -max, nbits_2 - 2), (min, false));
-        assert_eq!(max.mul_add_overflow(min, -max, nbits_2 - 3), (max, true));
         assert_eq!(
-            max.mul_add_overflow(min, -max + 1, nbits_2 - 3),
+            arith::overflowing_mul_add(max, min, -max, nbits_2 - 2),
+            (min, false)
+        );
+        assert_eq!(
+            arith::overflowing_mul_add(max, min, -max, nbits_2 - 3),
+            (max, true)
+        );
+        assert_eq!(
+            arith::overflowing_mul_add(max, min, -max + 1, nbits_2 - 3),
             (min, false)
         );
 
@@ -1372,94 +1288,202 @@ mod tests {
 
         let max = u128::MAX;
 
-        assert_eq!(max.mul_add_overflow(max, max, nbits_2), (max, false));
-        assert_eq!(max.mul_add_overflow(max, max, nbits_2 - 1), (0, true));
         assert_eq!(
-            max.mul_add_overflow(max, max - 1, nbits_2 - 1),
+            arith::overflowing_mul_add(max, max, max, nbits_2),
+            (max, false)
+        );
+        assert_eq!(
+            arith::overflowing_mul_add(max, max, max, nbits_2 - 1),
+            (0, true)
+        );
+        assert_eq!(
+            arith::overflowing_mul_add(max, max, max - 1, nbits_2 - 1),
             (max, false)
         );
 
         let (min, max) = (i128::MIN, i128::MAX);
 
-        assert_eq!(max.mul_add_overflow(max, max, nbits_2 - 2), (max, false));
-        assert_eq!(max.mul_add_overflow(max, max, nbits_2 - 3), (min, true));
         assert_eq!(
-            max.mul_add_overflow(max, max - 1, nbits_2 - 3),
+            arith::overflowing_mul_add(max, max, max, nbits_2 - 2),
+            (max, false)
+        );
+        assert_eq!(
+            arith::overflowing_mul_add(max, max, max, nbits_2 - 3),
+            (min, true)
+        );
+        assert_eq!(
+            arith::overflowing_mul_add(max, max, max - 1, nbits_2 - 3),
             (max, false)
         );
 
-        assert_eq!(min.mul_add_overflow(min, max, nbits_2 - 1), (max, false));
-        assert_eq!(min.mul_add_overflow(min, max, nbits_2 - 2), (min, true));
         assert_eq!(
-            min.mul_add_overflow(min, max - 1, nbits_2 - 2),
+            arith::overflowing_mul_add(min, min, max, nbits_2 - 1),
+            (max, false)
+        );
+        assert_eq!(
+            arith::overflowing_mul_add(min, min, max, nbits_2 - 2),
+            (min, true)
+        );
+        assert_eq!(
+            arith::overflowing_mul_add(min, min, max - 1, nbits_2 - 2),
             (max, false)
         );
 
-        assert_eq!(max.mul_add_overflow(min, -max, nbits_2 - 2), (min, false));
-        assert_eq!(max.mul_add_overflow(min, -max, nbits_2 - 3), (max, true));
         assert_eq!(
-            max.mul_add_overflow(min, -max + 1, nbits_2 - 3),
+            arith::overflowing_mul_add(max, min, -max, nbits_2 - 2),
+            (min, false)
+        );
+        assert_eq!(
+            arith::overflowing_mul_add(max, min, -max, nbits_2 - 3),
+            (max, true)
+        );
+        assert_eq!(
+            arith::overflowing_mul_add(max, min, -max + 1, nbits_2 - 3),
             (min, false)
         );
     }
 
     #[test]
-    fn mul_add_overflow_neg_frac_nbits() {
+    fn overflowing_mul_add_neg_frac_nbits() {
         let nbits = 64;
 
         let (zero, one, max) = (0u64, 1u64, u64::MAX);
 
-        assert_eq!(zero.mul_add_overflow(zero, max, -nbits), (max, false));
-        assert_eq!(one.mul_add_overflow(one, max, -nbits), (max, true));
         assert_eq!(
-            one.mul_add_overflow(one, zero, 1 - nbits),
+            arith::overflowing_mul_add(zero, zero, max, -nbits),
+            (max, false)
+        );
+        assert_eq!(
+            arith::overflowing_mul_add(one, one, max, -nbits),
+            (max, true)
+        );
+        assert_eq!(
+            arith::overflowing_mul_add(one, one, zero, 1 - nbits),
             (max - max / 2, false)
         );
-        assert_eq!(one.mul_add_overflow(one, max, 1 - nbits), (max / 2, true));
+        assert_eq!(
+            arith::overflowing_mul_add(one, one, max, 1 - nbits),
+            (max / 2, true)
+        );
 
         let (zero, one, min, max) = (0i64, 1i64, i64::MIN, i64::MAX);
 
-        assert_eq!(zero.mul_add_overflow(zero, max, -nbits), (max, false));
-        assert_eq!(one.mul_add_overflow(one, max, -nbits), (max, true));
-        assert_eq!(one.mul_add_overflow(one, -one, 1 - nbits), (max, false));
-        assert_eq!(one.mul_add_overflow(one, zero, 1 - nbits), (min, true));
+        assert_eq!(
+            arith::overflowing_mul_add(zero, zero, max, -nbits),
+            (max, false)
+        );
+        assert_eq!(
+            arith::overflowing_mul_add(one, one, max, -nbits),
+            (max, true)
+        );
+        assert_eq!(
+            arith::overflowing_mul_add(one, one, -one, 1 - nbits),
+            (max, false)
+        );
+        assert_eq!(
+            arith::overflowing_mul_add(one, one, zero, 1 - nbits),
+            (min, true)
+        );
 
-        assert_eq!((-one).mul_add_overflow(-one, max, -nbits), (max, true));
-        assert_eq!((-one).mul_add_overflow(-one, -one, 1 - nbits), (max, false));
-        assert_eq!((-one).mul_add_overflow(-one, zero, 1 - nbits), (min, true));
+        assert_eq!(
+            arith::overflowing_mul_add(-one, -one, max, -nbits),
+            (max, true)
+        );
+        assert_eq!(
+            arith::overflowing_mul_add(-one, -one, -one, 1 - nbits),
+            (max, false)
+        );
+        assert_eq!(
+            arith::overflowing_mul_add(-one, -one, zero, 1 - nbits),
+            (min, true)
+        );
 
-        assert_eq!(one.mul_add_overflow(-one, max, -nbits), (max, true));
-        assert_eq!(one.mul_add_overflow(-one, min, -nbits), (min, true));
-        assert_eq!(one.mul_add_overflow(-one, zero, 1 - nbits), (min, false));
-        assert_eq!(one.mul_add_overflow(-one, max, 1 - nbits), (-one, false));
+        assert_eq!(
+            arith::overflowing_mul_add(one, -one, max, -nbits),
+            (max, true)
+        );
+        assert_eq!(
+            arith::overflowing_mul_add(one, -one, min, -nbits),
+            (min, true)
+        );
+        assert_eq!(
+            arith::overflowing_mul_add(one, -one, zero, 1 - nbits),
+            (min, false)
+        );
+        assert_eq!(
+            arith::overflowing_mul_add(one, -one, max, 1 - nbits),
+            (-one, false)
+        );
 
         let nbits = 128;
 
         let (zero, one, max) = (0u128, 1u128, u128::MAX);
 
-        assert_eq!(zero.mul_add_overflow(zero, max, -nbits), (max, false));
-        assert_eq!(one.mul_add_overflow(one, max, -nbits), (max, true));
         assert_eq!(
-            one.mul_add_overflow(one, zero, 1 - nbits),
+            arith::overflowing_mul_add(zero, zero, max, -nbits),
+            (max, false)
+        );
+        assert_eq!(
+            arith::overflowing_mul_add(one, one, max, -nbits),
+            (max, true)
+        );
+        assert_eq!(
+            arith::overflowing_mul_add(one, one, zero, 1 - nbits),
             (max - max / 2, false)
         );
-        assert_eq!(one.mul_add_overflow(one, max, 1 - nbits), (max / 2, true));
+        assert_eq!(
+            arith::overflowing_mul_add(one, one, max, 1 - nbits),
+            (max / 2, true)
+        );
 
         let (zero, one, min, max) = (0i128, 1i128, i128::MIN, i128::MAX);
 
-        assert_eq!(zero.mul_add_overflow(zero, max, -nbits), (max, false));
-        assert_eq!(one.mul_add_overflow(one, max, -nbits), (max, true));
-        assert_eq!(one.mul_add_overflow(one, -one, 1 - nbits), (max, false));
-        assert_eq!(one.mul_add_overflow(one, zero, 1 - nbits), (min, true));
+        assert_eq!(
+            arith::overflowing_mul_add(zero, zero, max, -nbits),
+            (max, false)
+        );
+        assert_eq!(
+            arith::overflowing_mul_add(one, one, max, -nbits),
+            (max, true)
+        );
+        assert_eq!(
+            arith::overflowing_mul_add(one, one, -one, 1 - nbits),
+            (max, false)
+        );
+        assert_eq!(
+            arith::overflowing_mul_add(one, one, zero, 1 - nbits),
+            (min, true)
+        );
 
-        assert_eq!((-one).mul_add_overflow(-one, max, -nbits), (max, true));
-        assert_eq!((-one).mul_add_overflow(-one, -one, 1 - nbits), (max, false));
-        assert_eq!((-one).mul_add_overflow(-one, zero, 1 - nbits), (min, true));
+        assert_eq!(
+            arith::overflowing_mul_add(-one, -one, max, -nbits),
+            (max, true)
+        );
+        assert_eq!(
+            arith::overflowing_mul_add(-one, -one, -one, 1 - nbits),
+            (max, false)
+        );
+        assert_eq!(
+            arith::overflowing_mul_add(-one, -one, zero, 1 - nbits),
+            (min, true)
+        );
 
-        assert_eq!(one.mul_add_overflow(-one, max, -nbits), (max, true));
-        assert_eq!(one.mul_add_overflow(-one, min, -nbits), (min, true));
-        assert_eq!(one.mul_add_overflow(-one, zero, 1 - nbits), (min, false));
-        assert_eq!(one.mul_add_overflow(-one, max, 1 - nbits), (-one, false));
+        assert_eq!(
+            arith::overflowing_mul_add(one, -one, max, -nbits),
+            (max, true)
+        );
+        assert_eq!(
+            arith::overflowing_mul_add(one, -one, min, -nbits),
+            (min, true)
+        );
+        assert_eq!(
+            arith::overflowing_mul_add(one, -one, zero, 1 - nbits),
+            (min, false)
+        );
+        assert_eq!(
+            arith::overflowing_mul_add(one, -one, max, 1 - nbits),
+            (-one, false)
+        );
     }
 
     #[test]
