@@ -1,4 +1,4 @@
-// Copyright © 2018–2021 Trevor Spiteri
+// Copyright © 2018–2022 Trevor Spiteri
 
 // This library is free software: you can redistribute it and/or
 // modify it under the terms of either
@@ -14,8 +14,9 @@
 // <https://opensource.org/licenses/MIT>.
 
 use crate::{
-    helpers::{FloatHelper, FloatKind, IntHelper, Widest},
-    traits::Fixed,
+    float_helper,
+    helpers::{FloatKind, Widest},
+    int_helper::{self, IntFixed},
     types::extra::{LeEqU128, LeEqU16, LeEqU32, LeEqU64, LeEqU8},
     F128Bits, FixedI128, FixedI16, FixedI32, FixedI64, FixedI8, FixedU128, FixedU16, FixedU32,
     FixedU64, FixedU8,
@@ -24,22 +25,26 @@ use core::cmp::Ordering;
 use half::{bf16, f16};
 
 macro_rules! fixed_cmp_fixed {
-    ($Lhs:ident($LhsLeEqU:ident), $Rhs:ident($RhsLeEqU:ident)) => {
+    (
+        $Lhs:ident($LhsLeEqU:ident, $LhsInner:ident),
+        $Rhs:ident($RhsLeEqU:ident, $RhsInner:ident)
+    ) => {
         impl<FracLhs: $LhsLeEqU, FracRhs: $RhsLeEqU> PartialEq<$Rhs<FracRhs>> for $Lhs<FracLhs> {
             #[inline]
             fn eq(&self, rhs: &$Rhs<FracRhs>) -> bool {
-                let conv = rhs.to_bits().to_fixed_helper(
+                let conv = int_helper::$RhsInner::to_fixed_helper(
+                    rhs.to_bits(),
                     <$Rhs<FracRhs>>::FRAC_NBITS as i32,
                     Self::FRAC_NBITS,
                     Self::INT_NBITS,
                 );
                 let (rhs_is_neg, rhs_bits) = match conv.bits {
-                    Widest::Unsigned(bits) => (false, bits as <Self as Fixed>::Bits),
-                    Widest::Negative(bits) => (true, bits as <Self as Fixed>::Bits),
+                    Widest::Unsigned(bits) => (false, bits as $LhsInner),
+                    Widest::Negative(bits) => (true, bits as $LhsInner),
                 };
                 conv.dir == Ordering::Equal
                     && !conv.overflow
-                    && rhs_is_neg == rhs_bits.is_negative()
+                    && rhs_is_neg == int_helper::$LhsInner::is_negative(rhs_bits)
                     && rhs_bits == self.to_bits()
             }
         }
@@ -47,22 +52,24 @@ macro_rules! fixed_cmp_fixed {
         impl<FracLhs: $LhsLeEqU, FracRhs: $RhsLeEqU> PartialOrd<$Rhs<FracRhs>> for $Lhs<FracLhs> {
             #[inline]
             fn partial_cmp(&self, rhs: &$Rhs<FracRhs>) -> Option<Ordering> {
-                let rhs_is_neg = rhs.to_bits().is_negative();
-                match (self.to_bits().is_negative(), rhs_is_neg) {
+                let lhs_is_neg = int_helper::$LhsInner::is_negative(self.to_bits());
+                let rhs_is_neg = int_helper::$RhsInner::is_negative(rhs.to_bits());
+                match (lhs_is_neg, rhs_is_neg) {
                     (false, true) => return Some(Ordering::Greater),
                     (true, false) => return Some(Ordering::Less),
                     _ => {}
                 }
-                let conv = rhs.to_bits().to_fixed_helper(
+                let conv = int_helper::$RhsInner::to_fixed_helper(
+                    rhs.to_bits(),
                     <$Rhs<FracRhs>>::FRAC_NBITS as i32,
                     Self::FRAC_NBITS,
                     Self::INT_NBITS,
                 );
                 let rhs_bits = match conv.bits {
-                    Widest::Unsigned(bits) => bits as <Self as Fixed>::Bits,
-                    Widest::Negative(bits) => bits as <Self as Fixed>::Bits,
+                    Widest::Unsigned(bits) => bits as $LhsInner,
+                    Widest::Negative(bits) => bits as $LhsInner,
                 };
-                if conv.overflow || rhs_bits.is_negative() != rhs_is_neg {
+                if conv.overflow || int_helper::$LhsInner::is_negative(rhs_bits) != rhs_is_neg {
                     return if rhs_is_neg {
                         Some(Ordering::Greater)
                     } else {
@@ -74,22 +81,24 @@ macro_rules! fixed_cmp_fixed {
 
             #[inline]
             fn lt(&self, rhs: &$Rhs<FracRhs>) -> bool {
-                let rhs_is_neg = rhs.to_bits().is_negative();
-                match (self.to_bits().is_negative(), rhs_is_neg) {
+                let lhs_is_neg = int_helper::$LhsInner::is_negative(self.to_bits());
+                let rhs_is_neg = int_helper::$RhsInner::is_negative(rhs.to_bits());
+                match (lhs_is_neg, rhs_is_neg) {
                     (false, true) => return false,
                     (true, false) => return true,
                     _ => {}
                 }
-                let conv = rhs.to_bits().to_fixed_helper(
+                let conv = int_helper::$RhsInner::to_fixed_helper(
+                    rhs.to_bits(),
                     <$Rhs<FracRhs>>::FRAC_NBITS as i32,
                     Self::FRAC_NBITS,
                     Self::INT_NBITS,
                 );
                 let rhs_bits = match conv.bits {
-                    Widest::Unsigned(bits) => bits as <Self as Fixed>::Bits,
-                    Widest::Negative(bits) => bits as <Self as Fixed>::Bits,
+                    Widest::Unsigned(bits) => bits as $LhsInner,
+                    Widest::Negative(bits) => bits as $LhsInner,
                 };
-                if conv.overflow || rhs_bits.is_negative() != rhs_is_neg {
+                if conv.overflow || int_helper::$LhsInner::is_negative(rhs_bits) != rhs_is_neg {
                     return !rhs_is_neg;
                 }
                 let lhs_bits = self.to_bits();
@@ -119,26 +128,26 @@ macro_rules! fixed_cmp_int {
         impl<Frac: $LeEqU> PartialEq<$Int> for $Fix<Frac> {
             #[inline]
             fn eq(&self, rhs: &$Int) -> bool {
-                self.eq(&rhs.to_repr_fixed())
+                self.eq(&IntFixed(*rhs).fixed())
             }
         }
 
         impl<Frac: $LeEqU> PartialEq<$Fix<Frac>> for $Int {
             #[inline]
             fn eq(&self, rhs: &$Fix<Frac>) -> bool {
-                self.to_repr_fixed().eq(rhs)
+                IntFixed(*self).fixed().eq(rhs)
             }
         }
 
         impl<Frac: $LeEqU> PartialOrd<$Int> for $Fix<Frac> {
             #[inline]
             fn partial_cmp(&self, rhs: &$Int) -> Option<Ordering> {
-                self.partial_cmp(&rhs.to_repr_fixed())
+                self.partial_cmp(&IntFixed(*rhs).fixed())
             }
 
             #[inline]
             fn lt(&self, rhs: &$Int) -> bool {
-                self.lt(&rhs.to_repr_fixed())
+                self.lt(&IntFixed(*rhs).fixed())
             }
 
             #[inline]
@@ -160,12 +169,12 @@ macro_rules! fixed_cmp_int {
         impl<Frac: $LeEqU> PartialOrd<$Fix<Frac>> for $Int {
             #[inline]
             fn partial_cmp(&self, rhs: &$Fix<Frac>) -> Option<Ordering> {
-                self.to_repr_fixed().partial_cmp(rhs)
+                IntFixed(*self).fixed().partial_cmp(rhs)
             }
 
             #[inline]
             fn lt(&self, rhs: &$Fix<Frac>) -> bool {
-                self.to_repr_fixed().lt(rhs)
+                IntFixed(*self).fixed().lt(rhs)
             }
 
             #[inline]
@@ -187,21 +196,23 @@ macro_rules! fixed_cmp_int {
 }
 
 macro_rules! fixed_cmp_float {
-    ($Fix:ident($LeEqU:ident), $Float:ident) => {
+    ($Fix:ident($LeEqU:ident, $Inner:ident), $Float:ident) => {
         impl<Frac: $LeEqU> PartialEq<$Float> for $Fix<Frac> {
             #[inline]
             fn eq(&self, rhs: &$Float) -> bool {
-                let conv = match rhs.to_float_kind(Self::FRAC_NBITS, Self::INT_NBITS) {
+                let kind =
+                    float_helper::$Float::to_float_kind(*rhs, Self::FRAC_NBITS, Self::INT_NBITS);
+                let conv = match kind {
                     FloatKind::Finite { conv, .. } => conv,
                     _ => return false,
                 };
                 let (rhs_is_neg, rhs_bits) = match conv.bits {
-                    Widest::Unsigned(bits) => (false, bits as <Self as Fixed>::Bits),
-                    Widest::Negative(bits) => (true, bits as <Self as Fixed>::Bits),
+                    Widest::Unsigned(bits) => (false, bits as $Inner),
+                    Widest::Negative(bits) => (true, bits as $Inner),
                 };
                 conv.dir == Ordering::Equal
                     && !conv.overflow
-                    && rhs_is_neg == rhs_bits.is_negative()
+                    && rhs_is_neg == int_helper::$Inner::is_negative(rhs_bits)
                     && rhs_bits == self.to_bits()
             }
         }
@@ -216,8 +227,10 @@ macro_rules! fixed_cmp_float {
         impl<Frac: $LeEqU> PartialOrd<$Float> for $Fix<Frac> {
             #[inline]
             fn partial_cmp(&self, rhs: &$Float) -> Option<Ordering> {
-                let (rhs_is_neg, conv) = match rhs.to_float_kind(Self::FRAC_NBITS, Self::INT_NBITS)
-                {
+                let lhs_is_neg = int_helper::$Inner::is_negative(self.to_bits());
+                let kind =
+                    float_helper::$Float::to_float_kind(*rhs, Self::FRAC_NBITS, Self::INT_NBITS);
+                let (rhs_is_neg, conv) = match kind {
                     FloatKind::NaN => return None,
                     FloatKind::Infinite { neg } => {
                         return if neg {
@@ -228,16 +241,16 @@ macro_rules! fixed_cmp_float {
                     }
                     FloatKind::Finite { neg, conv } => (neg, conv),
                 };
-                match (self.to_bits().is_negative(), rhs_is_neg) {
+                match (lhs_is_neg, rhs_is_neg) {
                     (false, true) => return Some(Ordering::Greater),
                     (true, false) => return Some(Ordering::Less),
                     _ => {}
                 }
                 let rhs_bits = match conv.bits {
-                    Widest::Unsigned(bits) => bits as <Self as Fixed>::Bits,
-                    Widest::Negative(bits) => bits as <Self as Fixed>::Bits,
+                    Widest::Unsigned(bits) => bits as $Inner,
+                    Widest::Negative(bits) => bits as $Inner,
                 };
-                if conv.overflow || rhs_bits.is_negative() != rhs_is_neg {
+                if conv.overflow || int_helper::$Inner::is_negative(rhs_bits) != rhs_is_neg {
                     return if rhs_is_neg {
                         Some(Ordering::Greater)
                     } else {
@@ -249,23 +262,25 @@ macro_rules! fixed_cmp_float {
 
             #[inline]
             fn lt(&self, rhs: &$Float) -> bool {
-                let (rhs_is_neg, conv) = match rhs.to_float_kind(Self::FRAC_NBITS, Self::INT_NBITS)
-                {
+                let lhs_is_neg = int_helper::$Inner::is_negative(self.to_bits());
+                let kind =
+                    float_helper::$Float::to_float_kind(*rhs, Self::FRAC_NBITS, Self::INT_NBITS);
+                let (rhs_is_neg, conv) = match kind {
                     FloatKind::NaN => return false,
                     FloatKind::Infinite { neg } => return !neg,
                     FloatKind::Finite { neg, conv } => (neg, conv),
                 };
 
-                match (self.to_bits().is_negative(), rhs_is_neg) {
+                match (lhs_is_neg, rhs_is_neg) {
                     (false, true) => return false,
                     (true, false) => return true,
                     _ => {}
                 }
                 let rhs_bits = match conv.bits {
-                    Widest::Unsigned(bits) => bits as <Self as Fixed>::Bits,
-                    Widest::Negative(bits) => bits as <Self as Fixed>::Bits,
+                    Widest::Unsigned(bits) => bits as $Inner,
+                    Widest::Negative(bits) => bits as $Inner,
                 };
-                if conv.overflow || rhs_bits.is_negative() != rhs_is_neg {
+                if conv.overflow || int_helper::$Inner::is_negative(rhs_bits) != rhs_is_neg {
                     return !rhs_is_neg;
                 }
                 let lhs_bits = self.to_bits();
@@ -274,7 +289,7 @@ macro_rules! fixed_cmp_float {
 
             #[inline]
             fn le(&self, rhs: &$Float) -> bool {
-                !FloatHelper::is_nan(*rhs) && !rhs.lt(self)
+                !rhs.is_nan() && !rhs.lt(self)
             }
 
             #[inline]
@@ -284,7 +299,7 @@ macro_rules! fixed_cmp_float {
 
             #[inline]
             fn ge(&self, rhs: &$Float) -> bool {
-                !FloatHelper::is_nan(*rhs) && !self.lt(rhs)
+                !rhs.is_nan() && !self.lt(rhs)
             }
         }
 
@@ -296,23 +311,27 @@ macro_rules! fixed_cmp_float {
 
             #[inline]
             fn lt(&self, rhs: &$Fix<Frac>) -> bool {
-                let (lhs_is_neg, conv) =
-                    match self.to_float_kind(<$Fix<Frac>>::FRAC_NBITS, <$Fix<Frac>>::INT_NBITS) {
-                        FloatKind::NaN => return false,
-                        FloatKind::Infinite { neg } => return neg,
-                        FloatKind::Finite { neg, conv } => (neg, conv),
-                    };
-
-                match (lhs_is_neg, rhs.to_bits().is_negative()) {
+                let kind = float_helper::$Float::to_float_kind(
+                    *self,
+                    <$Fix<Frac>>::FRAC_NBITS,
+                    <$Fix<Frac>>::INT_NBITS,
+                );
+                let (lhs_is_neg, conv) = match kind {
+                    FloatKind::NaN => return false,
+                    FloatKind::Infinite { neg } => return neg,
+                    FloatKind::Finite { neg, conv } => (neg, conv),
+                };
+                let rhs_is_neg = int_helper::$Inner::is_negative(rhs.to_bits());
+                match (lhs_is_neg, rhs_is_neg) {
                     (false, true) => return false,
                     (true, false) => return true,
                     _ => {}
                 }
                 let lhs_bits = match conv.bits {
-                    Widest::Unsigned(bits) => bits as <$Fix<Frac> as Fixed>::Bits,
-                    Widest::Negative(bits) => bits as <$Fix<Frac> as Fixed>::Bits,
+                    Widest::Unsigned(bits) => bits as $Inner,
+                    Widest::Negative(bits) => bits as $Inner,
                 };
-                if conv.overflow || lhs_bits.is_negative() != lhs_is_neg {
+                if conv.overflow || int_helper::$Inner::is_negative(lhs_bits) != lhs_is_neg {
                     return lhs_is_neg;
                 }
                 let rhs_bits = rhs.to_bits();
@@ -321,7 +340,7 @@ macro_rules! fixed_cmp_float {
 
             #[inline]
             fn le(&self, rhs: &$Fix<Frac>) -> bool {
-                !FloatHelper::is_nan(*self) && !rhs.lt(self)
+                !self.is_nan() && !rhs.lt(self)
             }
 
             #[inline]
@@ -331,14 +350,14 @@ macro_rules! fixed_cmp_float {
 
             #[inline]
             fn ge(&self, rhs: &$Fix<Frac>) -> bool {
-                !FloatHelper::is_nan(*self) && !self.lt(rhs)
+                !self.is_nan() && !self.lt(rhs)
             }
         }
     };
 }
 
 macro_rules! fixed_cmp_all {
-    ($Fix:ident($LeEqU:ident)) => {
+    ($Fix:ident($LeEqU:ident, $Inner:ident)) => {
         impl<Frac: $LeEqU> Eq for $Fix<Frac> {}
 
         impl<Frac: $LeEqU> Ord for $Fix<Frac> {
@@ -348,16 +367,16 @@ macro_rules! fixed_cmp_all {
             }
         }
 
-        fixed_cmp_fixed! { $Fix($LeEqU), FixedI8(LeEqU8) }
-        fixed_cmp_fixed! { $Fix($LeEqU), FixedI16(LeEqU16) }
-        fixed_cmp_fixed! { $Fix($LeEqU), FixedI32(LeEqU32) }
-        fixed_cmp_fixed! { $Fix($LeEqU), FixedI64(LeEqU64) }
-        fixed_cmp_fixed! { $Fix($LeEqU), FixedI128(LeEqU128) }
-        fixed_cmp_fixed! { $Fix($LeEqU), FixedU8(LeEqU8) }
-        fixed_cmp_fixed! { $Fix($LeEqU), FixedU16(LeEqU16) }
-        fixed_cmp_fixed! { $Fix($LeEqU), FixedU32(LeEqU32) }
-        fixed_cmp_fixed! { $Fix($LeEqU), FixedU64(LeEqU64) }
-        fixed_cmp_fixed! { $Fix($LeEqU), FixedU128(LeEqU128) }
+        fixed_cmp_fixed! { $Fix($LeEqU, $Inner), FixedI8(LeEqU8, i8) }
+        fixed_cmp_fixed! { $Fix($LeEqU, $Inner), FixedI16(LeEqU16, i16) }
+        fixed_cmp_fixed! { $Fix($LeEqU, $Inner), FixedI32(LeEqU32, i32) }
+        fixed_cmp_fixed! { $Fix($LeEqU, $Inner), FixedI64(LeEqU64, i64) }
+        fixed_cmp_fixed! { $Fix($LeEqU, $Inner), FixedI128(LeEqU128, i128) }
+        fixed_cmp_fixed! { $Fix($LeEqU, $Inner), FixedU8(LeEqU8, u8) }
+        fixed_cmp_fixed! { $Fix($LeEqU, $Inner), FixedU16(LeEqU16, u16) }
+        fixed_cmp_fixed! { $Fix($LeEqU, $Inner), FixedU32(LeEqU32, u32) }
+        fixed_cmp_fixed! { $Fix($LeEqU, $Inner), FixedU64(LeEqU64, u64) }
+        fixed_cmp_fixed! { $Fix($LeEqU, $Inner), FixedU128(LeEqU128, u128) }
         fixed_cmp_int! { $Fix($LeEqU), i8 }
         fixed_cmp_int! { $Fix($LeEqU), i16 }
         fixed_cmp_int! { $Fix($LeEqU), i32 }
@@ -370,24 +389,24 @@ macro_rules! fixed_cmp_all {
         fixed_cmp_int! { $Fix($LeEqU), u64 }
         fixed_cmp_int! { $Fix($LeEqU), u128 }
         fixed_cmp_int! { $Fix($LeEqU), usize }
-        fixed_cmp_float! { $Fix($LeEqU), f16 }
-        fixed_cmp_float! { $Fix($LeEqU), bf16 }
-        fixed_cmp_float! { $Fix($LeEqU), f32 }
-        fixed_cmp_float! { $Fix($LeEqU), f64 }
-        fixed_cmp_float! { $Fix($LeEqU), F128Bits }
+        fixed_cmp_float! { $Fix($LeEqU, $Inner), f16 }
+        fixed_cmp_float! { $Fix($LeEqU, $Inner), bf16 }
+        fixed_cmp_float! { $Fix($LeEqU, $Inner), f32 }
+        fixed_cmp_float! { $Fix($LeEqU, $Inner), f64 }
+        fixed_cmp_float! { $Fix($LeEqU, $Inner), F128Bits }
     };
 }
 
-fixed_cmp_all! { FixedI8(LeEqU8) }
-fixed_cmp_all! { FixedI16(LeEqU16) }
-fixed_cmp_all! { FixedI32(LeEqU32) }
-fixed_cmp_all! { FixedI64(LeEqU64) }
-fixed_cmp_all! { FixedI128(LeEqU128) }
-fixed_cmp_all! { FixedU8(LeEqU8) }
-fixed_cmp_all! { FixedU16(LeEqU16) }
-fixed_cmp_all! { FixedU32(LeEqU32) }
-fixed_cmp_all! { FixedU64(LeEqU64) }
-fixed_cmp_all! { FixedU128(LeEqU128) }
+fixed_cmp_all! { FixedI8(LeEqU8, i8) }
+fixed_cmp_all! { FixedI16(LeEqU16, i16) }
+fixed_cmp_all! { FixedI32(LeEqU32, i32) }
+fixed_cmp_all! { FixedI64(LeEqU64, i64) }
+fixed_cmp_all! { FixedI128(LeEqU128, i128) }
+fixed_cmp_all! { FixedU8(LeEqU8, u8) }
+fixed_cmp_all! { FixedU16(LeEqU16, u16) }
+fixed_cmp_all! { FixedU32(LeEqU32, u32) }
+fixed_cmp_all! { FixedU64(LeEqU64, u64) }
+fixed_cmp_all! { FixedU128(LeEqU128, u128) }
 
 macro_rules! fixed_cmp {
     ($Fixed:ident($Inner:ty, $Len:ty, $bits_count:expr)) => {};
