@@ -1,4 +1,4 @@
-// Copyright © 2018–2021 Trevor Spiteri
+// Copyright © 2018–2022 Trevor Spiteri
 
 // This library is free software: you can redistribute it and/or
 // modify it under the terms of either
@@ -14,7 +14,7 @@
 // <https://opensource.org/licenses/MIT>.
 
 use crate::{
-    helpers::IntHelper,
+    int_helper::IntFixed,
     traits::{FromFixed, LosslessTryFrom, LossyFrom, ToFixed},
     types::extra::{
         Diff, IsLessOrEqual, LeEqU128, LeEqU16, LeEqU32, LeEqU64, LeEqU8, True, U0, U1, U127, U128,
@@ -142,7 +142,7 @@ macro_rules! convert_lossy {
             /// This conversion never fails (infallible) but may lose
             /// precision (lossy). Any fractional bits in the source
             /// that cannot be represented in the destination are
-            /// discarded, which rounds towards −∞.
+            /// discarded, which rounds towards &minus;∞.
             #[inline]
             fn lossy_from(src: $SrcU<FracSrc>) -> Self {
                 src.to_num()
@@ -161,7 +161,7 @@ macro_rules! convert_lossy {
             /// This conversion never fails (infallible) but may lose
             /// precision (lossy). Any fractional bits in the source
             /// that cannot be represented in the destination are
-            /// discarded, which rounds towards −∞.
+            /// discarded, which rounds towards &minus;∞.
             #[inline]
             fn lossy_from(src: $SrcI<FracSrc>) -> Self {
                 src.to_num()
@@ -180,7 +180,7 @@ macro_rules! convert_lossy {
             /// This conversion never fails (infallible) but may lose
             /// precision (lossy). Any fractional bits in the source
             /// that cannot be represented in the destination are
-            /// discarded, which rounds towards −∞.
+            /// discarded, which rounds towards &minus;∞.
             #[inline]
             fn lossy_from(src: $SrcU<FracSrc>) -> Self {
                 src.to_num()
@@ -606,7 +606,7 @@ macro_rules! fixed_to_int_lossy {
             ///
             /// This conversion never fails (infallible) but may lose
             /// precision (lossy). Any fractional bits in the source
-            /// are discarded, which rounds towards −∞.
+            /// are discarded, which rounds towards &minus;∞.
             #[inline]
             fn lossy_from(src: $SrcU<FracSrc>) -> Self {
                 src.to_num()
@@ -622,7 +622,7 @@ macro_rules! fixed_to_int_lossy {
             ///
             /// This conversion never fails (infallible) but may lose
             /// precision (lossy). Any fractional bits in the source
-            /// are discarded, which rounds towards −∞.
+            /// are discarded, which rounds towards &minus;∞.
             #[inline]
             fn lossy_from(src: $SrcI<FracSrc>) -> Self {
                 src.to_num()
@@ -638,7 +638,7 @@ macro_rules! fixed_to_int_lossy {
             ///
             /// This conversion never fails (infallible) but may lose
             /// precision (lossy). Any fractional bits in the source
-            /// are discarded, which rounds towards −∞.
+            /// are discarded, which rounds towards &minus;∞.
             #[inline]
             fn lossy_from(src: $SrcU<FracSrc>) -> Self {
                 src.to_num()
@@ -794,7 +794,7 @@ macro_rules! int_to_float_lossy_lossless {
                 /// nearest, with ties rounded to even.
                 #[inline]
                 fn lossy_from(src: $Int) -> $Lossy {
-                    Self::from_fixed(src.to_repr_fixed())
+                    Self::from_fixed(IntFixed(src).fixed())
                 }
             }
         )*
@@ -806,7 +806,7 @@ macro_rules! int_to_float_lossy_lossless {
                 /// and does not lose precision (lossless).
                 #[inline]
                 fn lossless_try_from(src: $Int) -> Option<$Lossless> {
-                    Some(Self::from_fixed(src.to_repr_fixed()))
+                    Some(Self::from_fixed(IntFixed(src).fixed()))
                 }
             }
 
@@ -817,7 +817,7 @@ macro_rules! int_to_float_lossy_lossless {
                 /// actually does not lose precision (lossless).
                 #[inline]
                 fn lossy_from(src: $Int) -> $Lossless {
-                    Self::from_fixed(src.to_repr_fixed())
+                    Self::from_fixed(IntFixed(src).fixed())
                 }
             }
         )*
