@@ -1,4 +1,4 @@
-// Copyright © 2018–2021 Trevor Spiteri
+// Copyright © 2018–2022 Trevor Spiteri
 
 // This library is free software: you can redistribute it and/or
 // modify it under the terms of either
@@ -13,76 +13,59 @@
 // <https://www.apache.org/licenses/LICENSE-2.0> and
 // <https://opensource.org/licenses/MIT>.
 
-use crate::{
-    helpers::{FloatKind, IntHelper, ToFixedHelper, ToFloatHelper, Widest},
-    F128Bits,
-};
-use core::cmp::Ordering;
-use half::{bf16, f16};
-
-pub trait FloatHelper: Copy {
-    type Bits: IntHelper;
-
-    const PREC: u32;
-    const EXP_BIAS: i32 = (1 << (Self::Bits::BITS - Self::PREC - 1)) - 1;
-    const EXP_MIN: i32 = 1 - Self::EXP_BIAS;
-    const EXP_MAX: i32 = Self::EXP_BIAS;
-    const SIGN_MASK: Self::Bits = Self::Bits::MSB;
-    const EXP_MASK: Self::Bits;
-    const MANT_MASK: Self::Bits;
-
-    fn is_nan(self) -> bool;
-
-    fn parts(self) -> (bool, i32, Self::Bits);
-
-    fn from_to_float_helper(val: ToFloatHelper, frac_bits: u32, int_bits: u32) -> Self;
-    fn to_float_kind(self, dst_frac_bits: u32, dst_int_bits: u32) -> FloatKind;
-}
-
-macro_rules! sealed_float {
-    ($Float:ident($Bits:ty, $IBits:ty, $prec:expr)) => {
-        impl FloatHelper for $Float {
-            type Bits = $Bits;
+macro_rules! make_helper {
+    ($Float:ident($Bits:ty, $IBits:ident, $prec:expr) $(; use $path:path)?) => {
+        #[allow(non_snake_case)]
+        pub mod $Float {
+            use crate::{
+                helpers::{FloatKind, ToFixedHelper, ToFloatHelper, Widest},
+                int_helper,
+            };
+            use core::cmp::Ordering;
+            $(use $path;)?
 
             const PREC: u32 = $prec;
-            const EXP_MASK: Self::Bits = !(Self::SIGN_MASK | Self::MANT_MASK);
-            const MANT_MASK: Self::Bits = (1 << (Self::PREC - 1)) - 1;
+            const EXP_BIAS: i32 = (1 << (<$Bits>::BITS - PREC - 1)) - 1;
+            const EXP_MIN: i32 = 1 - EXP_BIAS;
+            const EXP_MAX: i32 = EXP_BIAS;
+            pub const SIGN_MASK: $Bits = 1 << (<$Bits>::BITS - 1);
+            pub const EXP_MASK: $Bits = !(SIGN_MASK | MANT_MASK);
+            const MANT_MASK: $Bits = (1 << (PREC - 1)) - 1;
 
             #[inline]
-            fn is_nan(self) -> bool {
-                (self.to_bits() & !Self::SIGN_MASK) > Self::EXP_MASK
-            }
-
-            #[inline]
-            fn parts(self) -> (bool, i32, $Bits) {
-                let bits = self.to_bits();
-                let neg = bits & Self::SIGN_MASK != 0;
-                let biased_exp = (bits & Self::EXP_MASK) >> (Self::PREC - 1);
-                let exp = biased_exp as i32 - Self::EXP_BIAS;
-                let mant = bits & Self::MANT_MASK;
+            fn parts(val: $Float) -> (bool, i32, $Bits) {
+                let bits = val.to_bits();
+                let neg = bits & SIGN_MASK != 0;
+                let biased_exp = (bits & EXP_MASK) >> (PREC - 1);
+                let exp = biased_exp as i32 - EXP_BIAS;
+                let mant = bits & MANT_MASK;
 
                 (neg, exp, mant)
             }
 
             #[inline]
-            fn from_to_float_helper(val: ToFloatHelper, frac_bits: u32, int_bits: u32) -> $Float {
+            pub fn from_to_float_helper(
+                val: ToFloatHelper,
+                frac_bits: u32,
+                int_bits: u32,
+            ) -> $Float {
                 let fix_bits = frac_bits + int_bits;
 
-                let bits_sign = if val.neg { Self::SIGN_MASK } else { 0 };
+                let bits_sign = if val.neg { SIGN_MASK } else { 0 };
 
                 let extra_zeros = 128 - fix_bits;
                 let leading_zeros = val.abs.leading_zeros() - extra_zeros;
                 let signif_bits = fix_bits - leading_zeros;
                 if signif_bits == 0 {
-                    return Self::from_bits(bits_sign);
+                    return $Float::from_bits(bits_sign);
                 }
                 // remove leading zeros and implicit one
                 let mut mantissa = val.abs << leading_zeros << 1;
                 let exponent = int_bits as i32 - 1 - leading_zeros as i32;
-                let biased_exponent = if exponent > Self::EXP_MAX {
-                    return Self::from_bits(Self::EXP_MASK | bits_sign);
-                } else if exponent < Self::EXP_MIN {
-                    let lost_prec = Self::EXP_MIN - exponent;
+                let biased_exponent = if exponent > EXP_MAX {
+                    return $Float::from_bits(EXP_MASK | bits_sign);
+                } else if exponent < EXP_MIN {
+                    let lost_prec = EXP_MIN - exponent;
                     if lost_prec as u32 >= (int_bits + frac_bits) {
                         mantissa = 0;
                     } else {
@@ -92,11 +75,11 @@ macro_rules! sealed_float {
                     }
                     0
                 } else {
-                    (exponent + Self::EXP_MAX) as Self::Bits
+                    (exponent + EXP_MAX) as $Bits
                 };
                 // check for rounding
-                let round_up = (fix_bits >= Self::PREC) && {
-                    let shift = Self::PREC - 1;
+                let round_up = (fix_bits >= PREC) && {
+                    let shift = PREC - 1;
                     let mid_bit = !(!0 >> 1) >> (shift + extra_zeros);
                     let lower_bits = mid_bit - 1;
                     if mantissa & mid_bit == 0 {
@@ -108,25 +91,25 @@ macro_rules! sealed_float {
                         mantissa & (mid_bit << 1) != 0
                     }
                 };
-                let bits_exp = biased_exponent << (Self::PREC - 1);
-                let bits_mantissa = (if fix_bits >= Self::PREC - 1 {
-                    (mantissa >> (fix_bits - (Self::PREC - 1))) as Self::Bits
+                let bits_exp = biased_exponent << (PREC - 1);
+                let bits_mantissa = (if fix_bits >= PREC - 1 {
+                    (mantissa >> (fix_bits - (PREC - 1))) as $Bits
                 } else {
-                    (mantissa as Self::Bits) << (Self::PREC - 1 - fix_bits)
-                }) & !(!0 << (Self::PREC - 1));
+                    (mantissa as $Bits) << (PREC - 1 - fix_bits)
+                }) & !(!0 << (PREC - 1));
                 let mut bits_exp_mantissa = bits_exp | bits_mantissa;
                 if round_up {
                     bits_exp_mantissa += 1;
                 }
-                Self::from_bits(bits_sign | bits_exp_mantissa)
+                $Float::from_bits(bits_sign | bits_exp_mantissa)
             }
 
             #[inline]
-            fn to_float_kind(self, dst_frac_bits: u32, dst_int_bits: u32) -> FloatKind {
-                let prec = Self::PREC as i32;
+            pub fn to_float_kind(val: $Float, dst_frac_bits: u32, dst_int_bits: u32) -> FloatKind {
+                let prec = PREC as i32;
 
-                let (neg, exp, mut mantissa) = self.parts();
-                if exp > Self::EXP_MAX {
+                let (neg, exp, mut mantissa) = parts(val);
+                if exp > EXP_MAX {
                     if mantissa == 0 {
                         return FloatKind::Infinite { neg };
                     } else {
@@ -134,7 +117,7 @@ macro_rules! sealed_float {
                     };
                 }
                 // if not subnormal, add implicit bit
-                if exp >= Self::EXP_MIN {
+                if exp >= EXP_MIN {
                     mantissa |= 1 << (prec - 1);
                 }
                 if mantissa == 0 {
@@ -184,7 +167,12 @@ macro_rules! sealed_float {
                     mantissa = -mantissa;
                     dir = dir.reverse();
                 }
-                let mut conv = mantissa.to_fixed_helper(src_frac_bits, dst_frac_bits, dst_int_bits);
+                let mut conv = int_helper::$IBits::to_fixed_helper(
+                    mantissa,
+                    src_frac_bits,
+                    dst_frac_bits,
+                    dst_int_bits,
+                );
                 conv.dir = dir;
                 FloatKind::Finite { neg, conv }
             }
@@ -192,8 +180,8 @@ macro_rules! sealed_float {
     };
 }
 
-sealed_float! { f16(u16, i16, 11) }
-sealed_float! { bf16(u16, i16, 8) }
-sealed_float! { f32(u32, i32, 24) }
-sealed_float! { f64(u64, i64, 53) }
-sealed_float! { F128Bits(u128, i128, 113) }
+make_helper! { f16(u16, i16, 11); use half::f16 }
+make_helper! { bf16(u16, i16, 8); use half::bf16 }
+make_helper! { f32(u32, i32, 24) }
+make_helper! { f64(u64, i64, 53) }
+make_helper! { F128Bits(u128, i128, 113); use crate::F128Bits }
