@@ -1,4 +1,4 @@
-// Copyright © 2018–2021 Trevor Spiteri
+// Copyright © 2018–2022 Trevor Spiteri
 
 // This library is free software: you can redistribute it and/or
 // modify it under the terms of either
@@ -24,22 +24,25 @@ The [*fixed* crate] provides fixed-point numbers.
   * [`FixedI64`] and [`FixedU64`] are 64-bit fixed-point numbers.
   * [`FixedI128`] and [`FixedU128`] are 128-bit fixed-point numbers.
 
-An <i>n</i>-bit fixed-point number has <i>f</i> = `Frac` fractional bits where
-0 ≤ <i>f</i> ≤ <i>n</i>, and <i>n</i> − <i>f</i> integer bits. For example,
+An <i>n</i>-bit fixed-point number has <i>f</i>&nbsp;=&nbsp;`Frac` fractional
+bits where 0&nbsp;≤&nbsp;<i>f</i>&nbsp;≤&nbsp;<i>n</i>, and
+<i>n</i>&nbsp;&minus;&nbsp;<i>f</i> integer bits. For example,
 <code>[FixedI32]\<[U24]></code> is a 32-bit signed fixed-point number with
-<i>n</i> = 32 total bits, <i>f</i> = 24 fractional bits, and
-<i>n</i> − <i>f</i> = 8 integer bits. <code>[FixedI32]\<[U0]></code> behaves
-like [`i32`], and <code>[FixedU32]\<[U0]></code> behaves like [`u32`].
+<i>n</i>&nbsp;=&nbsp;32 total bits, <i>f</i>&nbsp;=&nbsp;24 fractional bits, and
+<i>n</i>&nbsp;&minus;&nbsp;<i>f</i>&nbsp;=&nbsp;8 integer bits.
+<code>[FixedI32]\<[U0]></code> behaves like [`i32`], and
+<code>[FixedU32]\<[U0]></code> behaves like [`u32`].
 
 The difference between any two successive representable numbers is constant
 throughout the possible range for a fixed-point number:
-<i>Δ</i> = 1/2<sup><i>f</i></sup>. When <i>f</i> = 0, like in
-<code>[FixedI32]\<[U0]></code>, <i>Δ</i> = 1 because representable numbers are
-integers, and the difference between two successive integers is 1. When
-<i>f</i> = <i>n</i>, <i>Δ</i> = 1/2<sup><i>n</i></sup> and the value lies in the
-range −0.5 ≤ <i>x</i> < 0.5 for signed numbers like
-<code>[FixedI32]\<[U32]></code>, and in the range 0 ≤ <i>x</i> < 1 for unsigned
-numbers like <code>[FixedU32]\<[U32]></code>.
+<i>Δ</i>&nbsp;=&nbsp;1/2<sup><i>f</i></sup>. When <i>f</i>&nbsp;=&nbsp;0, like
+in <code>[FixedI32]\<[U0]></code>, <i>Δ</i>&nbsp;=&nbsp;1 because representable
+numbers are integers, and the difference between two successive integers is 1.
+When <i>f</i>&nbsp;=&nbsp;<i>n</i>, <i>Δ</i>&nbsp;=&nbsp;1/2<sup><i>n</i></sup>
+and the value lies in the range &minus;0.5&nbsp;≤&nbsp;<i>x</i>&nbsp;<&nbsp;0.5
+for signed numbers like <code>[FixedI32]\<[U32]></code>, and in the range
+0&nbsp;≤&nbsp;<i>x</i>&nbsp;<&nbsp;1 for unsigned numbers like
+<code>[FixedU32]\<[U32]></code>.
 
 In version 1 the [*typenum* crate] is used for the fractional bit count `Frac`;
 the plan is to to have a major version 2 with [const generics] instead when the
@@ -126,7 +129,7 @@ combinations of integer and fractional bits adding up to a total of eight, 16,
 ```rust
 use fixed::types::{I4F4, I4F12};
 
-// −8 ≤ I4F4 < 8 with steps of 1/16 (~0.06)
+// -8 ≤ I4F4 < 8 with steps of 1/16 (~0.06)
 let a = I4F4::from_num(1);
 // multiplication and division by integers are possible
 let ans1 = a / 5 * 17;
@@ -134,7 +137,7 @@ let ans1 = a / 5 * 17;
 assert_eq!(ans1, I4F4::from_bits((3 << 4) + 3));
 assert_eq!(ans1.to_string(), "3.2");
 
-// −8 ≤ I4F12 < 8 with steps of 1/4096 (~0.0002)
+// -8 ≤ I4F12 < 8 with steps of 1/4096 (~0.0002)
 let wider_a = I4F12::from(a);
 let wider_ans = wider_a / 5 * 17;
 let ans2 = I4F4::from_num(wider_ans);
@@ -178,10 +181,10 @@ crate, add it as a dependency inside [*Cargo.toml*]:
 
 ```toml
 [dependencies]
-fixed = "1.10"
+fixed = "1.12"
 ```
 
-The *fixed* crate requires rustc version 1.53.0 or later.
+The *fixed* crate requires rustc version 1.57.0 or later.
 
 ## Optional features
 
@@ -205,7 +208,7 @@ To enable features, you can add the dependency like this to [*Cargo.toml*]:
 
 ```toml
 [dependencies.fixed]
-version = "1.10"
+version = "1.12"
 features = ["serde"]
 ```
 
@@ -216,7 +219,10 @@ are removed. The removal of experimental features would however require a minor
 version bump. Similarly, on a minor version bump, optional dependencies can be
 updated to an incompatible newer version.
 
- 1. `num-traits`, disabled by default. This implements some traits from the
+ 1. `borsh`, disabled by default. This implements serialization and
+    deserialization using the [*borsh* crate]. (The plan is to promote this to
+    an optional feature once the [*borsh* crate] reaches version 1.0.0.)
+ 2. `num-traits`, disabled by default. This implements some traits from the
     [*num-traits* crate]. (The plan is to promote this to an optional feature
     once the [*num-traits* crate] reaches version 1.0.0.)
 
@@ -248,6 +254,7 @@ shall be dual licensed as above, without any additional terms or conditions.
 
 [*Cargo.toml*]: https://doc.rust-lang.org/cargo/guide/dependencies.html
 [*arbitrary* crate]: https://crates.io/crates/arbitrary
+[*borsh* crate]: https://crates.io/crates/borsh
 [*cordic* crate]: https://crates.io/crates/cordic
 [*fixed* crate]: https://crates.io/crates/fixed
 [*fixed-macro* crate]: https://crates.io/crates/fixed-macro
@@ -289,10 +296,11 @@ shall be dual licensed as above, without any additional terms or conditions.
 */
 #![cfg_attr(not(feature = "std"), no_std)]
 #![warn(missing_docs)]
-#![doc(html_root_url = "https://docs.rs/fixed/~1.10")]
+#![warn(unsafe_op_in_unsafe_fn)]
+#![doc(html_root_url = "https://docs.rs/fixed/~1.12")]
 #![doc(test(attr(deny(warnings))))]
 #![cfg_attr(feature = "fail-on-warnings", deny(warnings))]
-#![allow(clippy::wrong_self_convention)]
+#![allow(clippy::question_mark)]
 
 #[cfg(all(not(feature = "std"), test))]
 extern crate std;
@@ -301,6 +309,8 @@ extern crate std;
 mod macros;
 
 mod arith;
+#[cfg(feature = "borsh")]
+mod borshize;
 mod cast;
 mod cmp;
 pub mod consts;
@@ -315,7 +325,10 @@ mod impl_arbitrary;
 mod impl_bytemuck;
 #[cfg(feature = "num-traits")]
 mod impl_num_traits;
+mod int256;
 mod int_helper;
+mod inv_lerp;
+mod lerp;
 mod log10;
 mod prim_traits;
 #[cfg(feature = "serde")]
@@ -323,20 +336,18 @@ mod serdeize;
 pub mod traits;
 pub mod types;
 mod unwrapped;
-mod wide_div;
 mod wrapping;
 
 #[cfg(feature = "num-traits")]
 pub use crate::impl_num_traits::RadixParseFixedError;
 use crate::{
-    arith::MulDivOverflow,
     from_str::FromStrRadix,
     log10::IntFracLog10,
     traits::{FromFixed, ToFixed},
     types::extra::{
-        IsLessOrEqual, LeEqU128, LeEqU16, LeEqU32, LeEqU64, LeEqU8, Sum, True, Unsigned, U12, U124,
-        U125, U126, U127, U128, U13, U14, U15, U16, U28, U29, U30, U31, U32, U4, U5, U6, U60, U61,
-        U62, U63, U64, U7, U8,
+        Diff, IsLessOrEqual, LeEqU128, LeEqU16, LeEqU32, LeEqU64, LeEqU8, Sum, True, Unsigned, U12,
+        U124, U125, U126, U127, U128, U13, U14, U15, U16, U28, U29, U30, U31, U32, U4, U5, U6, U60,
+        U61, U62, U63, U64, U7, U8,
     },
 };
 pub use crate::{from_str::ParseFixedError, unwrapped::Unwrapped, wrapping::Wrapping};
@@ -345,7 +356,7 @@ use core::{
     hash::{Hash, Hasher},
     marker::PhantomData,
     mem,
-    ops::Add,
+    ops::{Add, Sub},
 };
 
 /// A prelude to import useful traits.
@@ -397,70 +408,75 @@ macro_rules! fixed {
     (
         $description:expr,
         $Fixed:ident(
-            $Inner:ty, $LeEqU:tt, $s_nbits:expr,
+            $Inner:ident, $LeEqU:tt, $UNbits:ident, $s_nbits:expr,
             $s_nbits_m1:expr, $s_nbits_m2:expr, $s_nbits_m3:expr, $s_nbits_m4:expr
         ),
         $nbytes:expr, $bytes_val:expr, $rev_bytes_val:expr, $be_bytes:expr, $le_bytes:expr,
         $UFixed:ident, $UInner:ty, $Signedness:tt,
         $LeEqU_C0:tt, $LeEqU_C1:tt, $LeEqU_C2:tt, $LeEqU_C3:tt,
-        $Double:ident, $DoubleInner:ty, $s_nbits_2:expr, $HasDouble:tt
+        $HasDouble:tt, $Double:ident, $DoubleInner:ty, $s_nbits_2:expr
     ) => {
         fixed! {
             $description,
             $Fixed[stringify!($Fixed)](
-                $Inner[stringify!($Inner)], $LeEqU, $s_nbits,
+                $Inner[stringify!($Inner)], $LeEqU, $UNbits, $s_nbits,
                 $s_nbits_m1, $s_nbits_m2, $s_nbits_m3, $s_nbits_m4
             ),
             $nbytes, $bytes_val, $rev_bytes_val, $be_bytes, $le_bytes,
             $UFixed[stringify!($UFixed)], $UInner, $Signedness,
             $LeEqU_C0, $LeEqU_C1, $LeEqU_C2, $LeEqU_C3,
-            $Double, $DoubleInner, $s_nbits_2, $HasDouble
+            $HasDouble, $Double[stringify!($Double)], $DoubleInner, $s_nbits_2
         }
     };
     (
         $description:expr,
         $Fixed:ident[$s_fixed:expr](
-            $Inner:ty[$s_inner:expr], $LeEqU:tt, $s_nbits:expr,
+            $Inner:ident[$s_inner:expr], $LeEqU:tt, $UNbits:ident, $s_nbits:expr,
             $s_nbits_m1:expr, $s_nbits_m2:expr, $s_nbits_m3:expr, $s_nbits_m4:expr
         ),
         $nbytes:expr, $bytes_val:expr, $rev_bytes_val:expr, $be_bytes:expr, $le_bytes:expr,
         $UFixed:ident[$s_ufixed:expr], $UInner:ty, $Signedness:tt,
         $LeEqU_C0:tt, $LeEqU_C1:tt, $LeEqU_C2:tt, $LeEqU_C3:tt,
-        $Double:ident, $DoubleInner:ty, $s_nbits_2:expr, $HasDouble:tt
+        $HasDouble:tt, $Double:ident[$s_double:expr], $DoubleInner:ty, $s_nbits_2:expr
     ) => {
         comment! {
             $description, "-bit ",
             if_signed_unsigned!($Signedness, "signed", "unsigned"),
             " number with `Frac` fractional bits.
 
-The number has ", $s_nbits, " bits, of which <i>f</i> = `Frac` are fractional
-bits and ", $s_nbits, " − <i>f</i> are integer bits. The value <i>x</i> can lie
-in the range ",
+The number has ", $s_nbits, " bits, of which <i>f</i>&nbsp;=&nbsp;`Frac` are
+fractional bits and ", $s_nbits, "&nbsp;&minus;&nbsp;<i>f</i> are integer bits.
+The value <i>x</i> can lie in the range ",
             if_signed_unsigned!(
                 $Signedness,
-                concat!("−2<sup>", $s_nbits_m1, "</sup>/2<sup><i>f</i></sup>"),
+                concat!("&minus;2<sup>", $s_nbits_m1, "</sup>/2<sup><i>f</i></sup>"),
                 "0",
             ),
-            " ≤ <i>x</i> < 2<sup>",
+            "&nbsp;≤&nbsp;<i>x</i>&nbsp;<&nbsp;2<sup>",
             if_signed_unsigned!($Signedness, $s_nbits_m1, $s_nbits),
             "</sup>/2<sup><i>f</i></sup>. The difference between successive
-numbers is constant throughout the range: <i>Δ</i> = 1/2<sup><i>f</i></sup>.
+numbers is constant throughout the range: <i>Δ</i>&nbsp;=&nbsp;1/2<sup><i>f</i></sup>.
 
-For <code>", $s_fixed, "\\<[U0]></code>, <i>f</i> = 0 and <i>Δ</i> = 1, and the
-fixed-point number behaves like ",
+For <code>", $s_fixed, "\\<[U0]></code>, <i>f</i>&nbsp;=&nbsp;0 and
+<i>Δ</i>&nbsp;=&nbsp;1, and the fixed-point number behaves like ",
             if_signed_unsigned!($Signedness, "an", "a"),
             " [`", $s_inner, "`] with the value lying in the range ",
             if_signed_unsigned!(
                 $Signedness,
-                concat!("−2<sup>", $s_nbits_m1, "</sup>"),
+                concat!("&minus;2<sup>", $s_nbits_m1, "</sup>"),
                 "0",
             ),
-            " ≤ <i>x</i> < 2<sup>",
+            "&nbsp;≤&nbsp;<i>x</i>&nbsp;<&nbsp;2<sup>",
             if_signed_unsigned!($Signedness, $s_nbits_m1, $s_nbits),
             "</sup>. For <code>", $s_fixed, "\\<[U", $s_nbits, "]></code>,
-<i>f</i> = ", $s_nbits, " and <i>Δ</i> = 1/2<sup>", $s_nbits, "</sup>, and the
-value lies in the range ",
-            if_signed_unsigned!($Signedness, "−1/2 ≤ <i>x</i> < 1/2", "0 ≤ <i>x</i> < 1"),
+<i>f</i>&nbsp;=&nbsp;", $s_nbits, " and
+<i>Δ</i>&nbsp;=&nbsp;1/2<sup>", $s_nbits, "</sup>, and the value lies in the
+range ",
+            if_signed_unsigned!(
+                $Signedness,
+                "&minus;1/2&nbsp;≤&nbsp;<i>x</i>&nbsp;<&nbsp;1/2",
+                "0&nbsp;≤&nbsp;<i>x</i>&nbsp;<&nbsp;1",
+            ),
             ".
 
 `Frac` is an [`Unsigned`] as provided by the [*typenum* crate]; the plan is to
@@ -491,7 +507,7 @@ assert_eq!(two_point_75.to_string(), \"2.8\");
 ";
             #[repr(transparent)]
             pub struct $Fixed<Frac> {
-                bits: $Inner,
+                pub(crate) bits: $Inner,
                 phantom: PhantomData<Frac>,
             }
         }
@@ -527,10 +543,10 @@ assert_eq!(two_point_75.to_string(), \"2.8\");
 
         // inherent methods that do not require Frac bounds, some of which can thus be const
         fixed_no_frac! {
-            $Fixed[$s_fixed]($Inner[$s_inner], $LeEqU, $s_nbits, $s_nbits_m1),
+            $Fixed[$s_fixed]($Inner[$s_inner], $LeEqU, $UNbits, $s_nbits, $s_nbits_m1, $s_nbits_m2),
             $nbytes, $bytes_val, $rev_bytes_val, $be_bytes, $le_bytes,
             $UFixed[$s_ufixed], $UInner, $Signedness,
-            $Double, $DoubleInner, $s_nbits_2, $HasDouble
+            $HasDouble, $Double[$s_double], $DoubleInner, $s_nbits_2
         }
         // inherent methods that require Frac bounds, and cannot be const
         fixed_frac! {
@@ -547,41 +563,41 @@ assert_eq!(two_point_75.to_string(), \"2.8\");
 
 fixed! {
     "An eight",
-    FixedU8(u8, LeEqU8, "8", "7", "6", "5", "4"),
+    FixedU8(u8, LeEqU8, U8, "8", "7", "6", "5", "4"),
     1, "0x12", "0x12", "[0x12]", "[0x12]",
     FixedU8, u8, Unsigned,
     U8, U7, U6, U5,
-    FixedU16, u16, "16", True
+    True, FixedU16, u16, "16"
 }
 fixed! {
     "A 16",
-    FixedU16(u16, LeEqU16, "16", "15", "14", "13", "12"),
+    FixedU16(u16, LeEqU16, U16, "16", "15", "14", "13", "12"),
     2, "0x1234", "0x3412", "[0x12, 0x34]", "[0x34, 0x12]",
     FixedU16, u16, Unsigned,
     U16, U15, U14, U13,
-    FixedU32, u32, "32", True
+    True, FixedU32, u32, "32"
 }
 fixed! {
     "A 32",
-    FixedU32(u32, LeEqU32, "32", "31", "30", "29", "28"),
+    FixedU32(u32, LeEqU32, U32, "32", "31", "30", "29", "28"),
     4, "0x1234_5678", "0x7856_3412", "[0x12, 0x34, 0x56, 0x78]", "[0x78, 0x56, 0x34, 0x12]",
     FixedU32, u32, Unsigned,
     U32, U31, U30, U29,
-    FixedU64, u64, "64", True
+    True, FixedU64, u64, "64"
 }
 fixed! {
     "A 64",
-    FixedU64(u64, LeEqU64, "64", "63", "62", "61", "60"),
+    FixedU64(u64, LeEqU64, U64, "64", "63", "62", "61", "60"),
     8, "0x1234_5678_9ABC_DE0F", "0x0FDE_BC9A_7856_3412",
     "[0x12, 0x34, 0x56, 0x78, 0x9A, 0xBC, 0xDE, 0x0F]",
     "[0x0F, 0xDE, 0xBC, 0x9A, 0x78, 0x56, 0x34, 0x12]",
     FixedU64, u64, Unsigned,
     U64, U63, U62, U61,
-    FixedU128, u128, "128", True
+    True, FixedU128, u128, "128"
 }
 fixed! {
     "A 128",
-    FixedU128(u128, LeEqU128, "128", "127", "126", "125", "124"),
+    FixedU128(u128, LeEqU128, U128, "128", "127", "126", "125", "124"),
     16, "0x1234_5678_9ABC_DEF0_0102_0304_0506_0708",
     "0x0807_0605_0403_0201_F0DE_BC9A_7856_3412",
     "[0x12, 0x34, 0x56, 0x78, 0x9A, 0xBC, 0xDE, 0xF0, \
@@ -590,45 +606,45 @@ fixed! {
      0xF0, 0xDE, 0xBC, 0x9A, 0x78, 0x56, 0x34, 0x12]",
     FixedU128, u128, Unsigned,
     U128, U127, U126, U125,
-    FixedU128, u128, "128", False
+    False, FixedU128, u128, "128"
 }
 fixed! {
     "An eight",
-    FixedI8(i8, LeEqU8, "8", "7", "6", "5", "4"),
+    FixedI8(i8, LeEqU8, U8, "8", "7", "6", "5", "4"),
     1, "0x12", "0x12", "[0x12]", "[0x12]",
     FixedU8, u8, Signed,
     U7, U6, U5, U4,
-    FixedI16, i16, "16", True
+    True, FixedI16, i16, "16"
 }
 fixed! {
     "A 16",
-    FixedI16(i16, LeEqU16, "16", "15", "14", "13", "12"),
+    FixedI16(i16, LeEqU16, U16,"16", "15", "14", "13", "12"),
     2, "0x1234", "0x3412", "[0x12, 0x34]", "[0x34, 0x12]",
     FixedU16, u16, Signed,
     U15, U14, U13, U12,
-    FixedI32, i32, "32", True
+    True, FixedI32, i32, "32"
 }
 fixed! {
     "A 32",
-    FixedI32(i32, LeEqU32, "32", "31", "30", "29", "28"),
+    FixedI32(i32, LeEqU32, U32, "32", "31", "30", "29", "28"),
     4, "0x1234_5678", "0x7856_3412", "[0x12, 0x34, 0x56, 0x78]", "[0x78, 0x56, 0x34, 0x12]",
     FixedU32, u32, Signed,
     U31, U30, U29, U28,
-    FixedI64, i64, "64", True
+    True, FixedI64, i64, "64"
 }
 fixed! {
     "A 64",
-    FixedI64(i64, LeEqU64, "64", "63", "62", "61", "60"),
+    FixedI64(i64, LeEqU64, U64, "64", "63", "62", "61", "60"),
     8, "0x1234_5678_9ABC_DE0F", "0x0FDE_BC9A_7856_3412",
     "[0x12, 0x34, 0x56, 0x78, 0x9A, 0xBC, 0xDE, 0x0F]",
     "[0x0F, 0xDE, 0xBC, 0x9A, 0x78, 0x56, 0x34, 0x12]",
     FixedU64, u64, Signed,
     U63, U62, U61, U60,
-    FixedI128, i128, "128", True
+    True, FixedI128, i128, "128"
 }
 fixed! {
     "A 128",
-    FixedI128(i128, LeEqU128, "128", "127", "126", "125", "124"),
+    FixedI128(i128, LeEqU128, U128, "128", "127", "126", "125", "124"),
     16, "0x1234_5678_9ABC_DEF0_0102_0304_0506_0708",
     "0x0807_0605_0403_0201_F0DE_BC9A_7856_3412",
     "[0x12, 0x34, 0x56, 0x78, 0x9A, 0xBC, 0xDE, 0xF0, \
@@ -637,7 +653,7 @@ fixed! {
      0xF0, 0xDE, 0xBC, 0x9A, 0x78, 0x56, 0x34, 0x12]",
     FixedU128, u128, Signed,
     U127, U126, U125, U124,
-    FixedI128, i128, "128", False
+    False, FixedI128, i128, "128"
 }
 
 /// The bit representation of a *binary128* floating-point number (`f128`).
@@ -672,10 +688,16 @@ impl F128Bits {
     pub(crate) fn to_bits(self) -> u128 {
         self.0
     }
+
     #[inline]
     pub(crate) fn from_bits(bits: u128) -> F128Bits {
         F128Bits(bits)
     }
+
+    #[inline]
+    pub(crate) fn is_nan(self) -> bool {
+        (self.to_bits() & !float_helper::F128Bits::SIGN_MASK) > float_helper::F128Bits::EXP_MASK
+    }
 }
 
 /// Defines constant fixed-point numbers from integer expressions.
