@@ -1,4 +1,4 @@
-// Copyright © 2018–2021 Trevor Spiteri
+// Copyright © 2018–2022 Trevor Spiteri
 
 // This library is free software: you can redistribute it and/or
 // modify it under the terms of either
@@ -16,7 +16,8 @@
 macro_rules! fixed_frac {
     (
         $Fixed:ident[$s_fixed:expr](
-            $Inner:ty[$s_inner:expr], $LeEqU:tt, $s_nbits:expr, $s_nbits_m1:expr, $s_nbits_m4:expr
+            $Inner:ident[$s_inner:expr], $LeEqU:tt, $s_nbits:expr,
+            $s_nbits_m1:expr, $s_nbits_m4:expr
         ),
         $UFixed:ident, $UInner:ty, $Signedness:tt
     ) => {
@@ -72,7 +73,7 @@ assert_eq!(Fix::FRAC_NBITS, 6);
 
 # Panics
 
-Panics if the fixed-point number is ", if_signed_unsigned!($Signedness, "≤ 0", "zero"), ".
+Panics if the fixed-point number is ", if_signed_unsigned!($Signedness, "≤&nbsp;0", "zero"), ".
 
 # Examples
 
@@ -96,7 +97,7 @@ assert_eq!(Fix::from_num(0.1875).int_log2(), -3);
 
 # Panics
 
-Panics if the fixed-point number is ", if_signed_unsigned!($Signedness, "≤ 0", "zero"), ".
+Panics if the fixed-point number is ", if_signed_unsigned!($Signedness, "≤&nbsp;0", "zero"), ".
 
 # Examples
 
@@ -120,7 +121,7 @@ assert_eq!(", $s_fixed, "::<U6>::from_num(0.09375).int_log10(), -2);
             comment! {
                 "Checked integer base-2 logarithm, rounded down.
 Returns the logarithm or [`None`] if the fixed-point number is
-", if_signed_unsigned!($Signedness, "≤ 0", "zero"), ".
+", if_signed_unsigned!($Signedness, "≤&nbsp;0", "zero"), ".
 
 # Examples
 
@@ -147,7 +148,7 @@ assert_eq!(Fix::from_num(0.1875).checked_int_log2(), Some(-3));
             comment! {
                 "Checked integer base-10 logarithm, rounded down.
 Returns the logarithm or [`None`] if the fixed-point number is
-", if_signed_unsigned!($Signedness, "≤ 0", "zero"), ".
+", if_signed_unsigned!($Signedness, "≤&nbsp;0", "zero"), ".
 
 # Examples
 
@@ -191,11 +192,11 @@ When debug assertions are enabled, this method panics
   * if the value is positive and the fixed-point number has zero
     or one integer bits such that it cannot hold the value 1.
   * if the value is negative and the fixed-point number has zero
-    integer bits, such that it cannot hold the value −1.
+    integer bits, such that it cannot hold the value &minus;1.
 
 When debug assertions are not enabled, the wrapped value can be
 returned in those cases, but it is not considered a breaking change if
-in the future it panics; using this method when 1 and −1 cannot be
+in the future it panics; using this method when 1 and &minus;1 cannot be
 represented is almost certainly a bug.
 
 # Examples
@@ -209,6 +210,7 @@ assert_eq!(Fix::from_num(-5).signum(), -1);
 ```
 ";
                     #[inline]
+                    #[must_use]
                     pub fn signum(self) -> $Fixed<Frac> {
                         match self.to_bits().cmp(&0) {
                             Ordering::Equal => Self::ZERO,
@@ -230,7 +232,7 @@ When debug assertions are enabled, this method also panics if the
 reciprocal overflows. When debug assertions are not enabled, the
 wrapped value can be returned, but it is not considered a breaking
 change if in the future it panics; if wrapping is required use
-[`wrapping_recip`] instead. 
+[`wrapping_recip`] instead.
 
 # Examples
 
@@ -242,9 +244,12 @@ assert_eq!(Fix::from_num(2).recip(), Fix::from_num(0.5));
 
 [`wrapping_recip`]: Self::wrapping_recip
 ";
-                #[inline] pub fn recip(self) -> $Fixed<Frac> {
+                #[inline]
+                #[must_use]
+                pub fn recip(self) -> $Fixed<Frac> {
                     let (ans, overflow) = self.overflowing_recip();
-                    debug_assert!(!overflow, "overflow"); ans
+                    debug_assert!(!overflow, "overflow");
+                    ans
                 }
             }
 
@@ -307,7 +312,7 @@ Panics if the divisor is zero.
                     $Signedness;
                     "When debug assertions are enabled, this method
 also panics if the division overflows. Overflow can only occur when
-dividing the minimum value by −1. When debug assertions are not
+dividing the minimum value by &minus;1. When debug assertions are not
 enabled, the wrapped value can be returned, but it is not considered a
 breaking change if in the future it panics; if wrapping is required
 use [`wrapping_div_euclid_int`] instead.
@@ -382,7 +387,7 @@ assert_eq!(acc, 5);
                 if_signed_else_empty_str! {
                     $Signedness;
                     "
-// MAX × 1.5 − MAX = MAX / 2, which does not overflow
+// MAX × 1.5 - MAX = MAX / 2, which does not overflow
 acc = -Fix::MAX;
 acc.mul_acc(Fix::MAX, Fix::from_num(1.5));
 assert_eq!(acc, Fix::MAX / 2);
@@ -398,7 +403,8 @@ assert_eq!(acc, Fix::MAX / 2);
                     a: $Fixed<AFrac>,
                     b: $Fixed<BFrac>,
                 ) {
-                    let (ans, overflow) = a.to_bits().mul_add_overflow(
+                    let (ans, overflow) = arith::overflowing_mul_add(
+                        a.to_bits(),
                         b.to_bits(),
                         self.to_bits(),
                         AFrac::I32 + BFrac::I32 - Frac::I32,
@@ -438,6 +444,60 @@ assert_eq!(Fix::from_num(7.5).rem_euclid_int(2), Fix::from_num(1.5));
                 }
             }
 
+            comment! {
+                "Linear interpolation between `start` and `end`.
+
+Returns
+`start`&nbsp;+&nbsp;`self`&nbsp;×&nbsp;(`end`&nbsp;&minus;&nbsp;`start`). This
+is `start` when `self`&nbsp;=&nbsp;0, `end` when `self`&nbsp;=&nbsp;1, and
+linear interpolation for all other values of `self`. Linear extrapolation is
+performed if `self` is not in the range 0&nbsp;≤&nbsp;<i>x</i>&nbsp;≤&nbsp;1.
+
+# Panics
+
+When debug assertions are enabled, this method panics if the result overflows.
+When debug assertions are not enabled, the wrapped value can be returned, but it
+is not considered a breaking change if in the future it panics; if wrapping is
+required use [`wrapping_lerp`] instead.
+
+# Examples
+
+```rust
+use fixed::{types::extra::U4, ", $s_fixed, "};
+type Fix = ", $s_fixed, "<U4>;
+let start = Fix::from_num(2);
+let end = Fix::from_num(3.5);
+",
+                if_signed_else_empty_str! {
+                    $Signedness;
+                    "assert_eq!(Fix::from_num(-1.0).lerp(start, end), 0.5);
+",
+                },
+                "assert_eq!(Fix::from_num(0.0).lerp(start, end), 2);
+assert_eq!(Fix::from_num(0.5).lerp(start, end), 2.75);
+assert_eq!(Fix::from_num(1.0).lerp(start, end), 3.5);
+assert_eq!(Fix::from_num(2.0).lerp(start, end), 5);
+```
+
+[`wrapping_lerp`]: Self::wrapping_lerp
+";
+                #[inline]
+                pub fn lerp<RangeFrac>(
+                    self,
+                    start: $Fixed<RangeFrac>,
+                    end: $Fixed<RangeFrac>,
+                ) -> $Fixed<RangeFrac> {
+                    let (ans, overflow) = lerp::$Inner(
+                        self.to_bits(),
+                        start.to_bits(),
+                        end.to_bits(),
+                        Frac::U32,
+                    );
+                    debug_assert!(!overflow, "overflow");
+                    $Fixed::from_bits(ans)
+                }
+            }
+
             if_signed! {
                 $Signedness;
                 comment! {
@@ -448,7 +508,7 @@ Overflow can only occur
   * if the value is positive and the fixed-point number has zero
     or one integer bits such that it cannot hold the value 1.
   * if the value is negative and the fixed-point number has zero
-    integer bits, such that it cannot hold the value −1.
+    integer bits, such that it cannot hold the value &minus;1.
 
 # Examples
 
@@ -495,7 +555,7 @@ assert_eq!(Fix::MAX.checked_mul(Fix::from_num(2)), None);
                 #[inline]
                 #[must_use = "this returns the result of the operation, without modifying the original"]
                 pub fn checked_mul(self, rhs: $Fixed<Frac>) -> Option<$Fixed<Frac>> {
-                    match self.to_bits().mul_overflow(rhs.to_bits(), Frac::U32) {
+                    match arith::overflowing_mul(self.to_bits(), rhs.to_bits(), Frac::U32) {
                         (ans, false) => Some(Self::from_bits(ans)),
                         (_, true) => None,
                     }
@@ -521,7 +581,7 @@ assert_eq!(Fix::MAX.checked_div(Fix::ONE / 2), None);
                     if rhs.to_bits() == 0 {
                         return None;
                     }
-                    match self.to_bits().div_overflow(rhs.to_bits(), Frac::U32) {
+                    match arith::overflowing_div(self.to_bits(), rhs.to_bits(), Frac::U32) {
                         (ans, false) => Some(Self::from_bits(ans)),
                         (_, true) => None,
                     }
@@ -634,7 +694,7 @@ assert_eq!(acc, Fix::DELTA);
                 if_signed_else_empty_str! {
                     $Signedness;
                     "
-// MAX × 1.5 − MAX = MAX / 2, which does not overflow
+// MAX × 1.5 - MAX = MAX / 2, which does not overflow
 acc = -Fix::MAX;
 let check = acc.checked_mul_acc(Fix::MAX, Fix::from_num(1.5));
 assert_eq!(check, Some(()));
@@ -652,7 +712,8 @@ assert_eq!(acc, Fix::MAX / 2);
                     a: $Fixed<AFrac>,
                     b: $Fixed<BFrac>,
                 ) -> Option<()> {
-                    let (ans, overflow) = a.to_bits().mul_add_overflow(
+                    let (ans, overflow) = arith::overflowing_mul_add(
+                        a.to_bits(),
                         b.to_bits(),
                         self.to_bits(),
                         AFrac::I32 + BFrac::I32 - Frac::I32,
@@ -770,7 +831,7 @@ assert_eq!(Fix::from_num(7.5).checked_rem_euclid_int(0), None);
                 if_signed_else_empty_str! {
                     $Signedness;
                     "assert_eq!(Fix::from_num(-7.5).checked_rem_euclid_int(2), Some(Fix::from_num(0.5)));
-// −8 ≤ Fix < 8, so the answer 12.5 overflows
+// -8 ≤ Fix < 8, so the answer 12.5 overflows
 assert_eq!(Fix::from_num(-7.5).checked_rem_euclid_int(20), None);
 ",
                 },
@@ -808,6 +869,38 @@ assert_eq!(Fix::from_num(-7.5).checked_rem_euclid_int(20), None);
                 }
             }
 
+            comment! {
+                "Checked linear interpolation between `start` and `end`. Returns
+[`None`] on overflow.
+
+The interpolted value is
+`start`&nbsp;+&nbsp;`self`&nbsp;×&nbsp;(`end`&nbsp;&minus;&nbsp;`start`). This
+is `start` when `self`&nbsp;=&nbsp;0, `end` when `self`&nbsp;=&nbsp;1, and
+linear interpolation for all other values of `self`. Linear extrapolation is
+performed if `self` is not in the range 0&nbsp;≤&nbsp;<i>x</i>&nbsp;≤&nbsp;1.
+
+# Examples
+
+```rust
+use fixed::{types::extra::U4, ", $s_fixed, "};
+type Fix = ", $s_fixed, "<U4>;
+assert_eq!(Fix::from_num(0.5).checked_lerp(Fix::ZERO, Fix::MAX), Some(Fix::MAX / 2));
+assert_eq!(Fix::from_num(1.5).checked_lerp(Fix::ZERO, Fix::MAX), None);
+```
+";
+                #[inline]
+                pub fn checked_lerp<RangeFrac>(
+                    self,
+                    start: $Fixed<RangeFrac>,
+                    end: $Fixed<RangeFrac>,
+                ) -> Option<$Fixed<RangeFrac>> {
+                    match lerp::$Inner(self.to_bits(), start.to_bits(), end.to_bits(), Frac::U32) {
+                        (bits, false) => Some($Fixed::from_bits(bits)),
+                        (_, true) => None,
+                    }
+                }
+            }
+
             if_signed! {
                 $Signedness;
                 comment! {
@@ -818,7 +911,7 @@ Overflow can only occur
   * if the value is positive and the fixed-point number has zero
     or one integer bits such that it cannot hold the value 1.
   * if the value is negative and the fixed-point number has zero
-    integer bits, such that it cannot hold the value −1.
+    integer bits, such that it cannot hold the value &minus;1.
 
 # Examples
 
@@ -840,6 +933,7 @@ assert_eq!(ZeroIntBits::from_num(-0.5).saturating_signum(), ZeroIntBits::MIN);
 ```
 ";
                     #[inline]
+                    #[must_use]
                     pub fn saturating_signum(self) -> $Fixed<Frac> {
                         match self.to_bits().cmp(&0) {
                             Ordering::Equal => Self::ZERO,
@@ -865,7 +959,7 @@ assert_eq!(Fix::MAX.saturating_mul(Fix::from_num(2)), Fix::MAX);
                 #[inline]
                 #[must_use = "this returns the result of the operation, without modifying the original"]
                 pub fn saturating_mul(self, rhs: $Fixed<Frac>) -> $Fixed<Frac> {
-                    match self.to_bits().mul_overflow(rhs.to_bits(), Frac::U32) {
+                    match arith::overflowing_mul(self.to_bits(), rhs.to_bits(), Frac::U32) {
                         (ans, false) => Self::from_bits(ans),
                         (_, true) => {
                             if (self < 0) != (rhs < 0) {
@@ -898,7 +992,7 @@ assert_eq!(Fix::MAX.saturating_div(one_half), Fix::MAX);
                 #[inline]
                 #[must_use = "this returns the result of the operation, without modifying the original"]
                 pub fn saturating_div(self, rhs: $Fixed<Frac>) -> $Fixed<Frac> {
-                    match self.to_bits().div_overflow(rhs.to_bits(), Frac::U32) {
+                    match arith::overflowing_div(self.to_bits(), rhs.to_bits(), Frac::U32) {
                         (ans, false) => Self::from_bits(ans),
                         (_, true) => {
                             if (self < 0) != (rhs < 0) {
@@ -935,6 +1029,7 @@ assert_eq!(Fix::from_num(0.25).saturating_recip(), Fix::MAX);
                 "```
 ";
                 #[inline]
+                #[must_use]
                 pub fn saturating_recip(self) -> $Fixed<Frac> {
                     match self.overflowing_recip() {
                         (ans, false) => ans,
@@ -976,9 +1071,7 @@ assert_eq!(Fix::MIN.saturating_div_euclid(Fix::from_num(0.25)), Fix::MIN);
                 #[inline]
                 #[must_use = "this returns the result of the operation, without modifying the original"]
                 pub fn saturating_div_euclid(self, rhs: $Fixed<Frac>) -> $Fixed<Frac> {
-                    if rhs.to_bits() == 0 {
-                        panic!("division by zero");
-                    }
+                    assert!(rhs.to_bits() != 0, "division by zero");
                     self.checked_div_euclid(rhs).unwrap_or_else(|| {
                         if (self.to_bits() > 0) == (rhs.to_bits() > 0) {
                             Self::MAX
@@ -995,7 +1088,7 @@ assert_eq!(Fix::MIN.saturating_div_euclid(Fix::from_num(0.25)), Fix::MIN);
                     $Signedness,
                     ", saturating on overflow.
 
-Overflow can only occur when dividing the minimum value by −1.",
+Overflow can only occur when dividing the minimum value by &minus;1.",
                     ".
 
 Can never overflow for unsigned values.",
@@ -1063,7 +1156,7 @@ assert_eq!(acc, Fix::MAX);
                 if_signed_else_empty_str! {
                     $Signedness;
                     "
-// MAX × 1.5 − MAX = MAX / 2, which does not overflow
+// MAX × 1.5 - MAX = MAX / 2, which does not overflow
 acc = -Fix::MAX;
 acc.saturating_mul_acc(Fix::MAX, Fix::from_num(1.5));
 assert_eq!(acc, Fix::MAX / 2);
@@ -1077,7 +1170,8 @@ assert_eq!(acc, Fix::MAX / 2);
                     a: $Fixed<AFrac>,
                     b: $Fixed<BFrac>,
                 ) {
-                    let (ans, overflow) = a.to_bits().mul_add_overflow(
+                    let (ans, overflow) = arith::overflowing_mul_add(
+                        a.to_bits(),
                         b.to_bits(),
                         self.to_bits(),
                         AFrac::I32 + BFrac::I32 - Frac::I32,
@@ -1124,7 +1218,7 @@ assert_eq!(Fix::from_num(7.5).saturating_rem_euclid_int(2), Fix::from_num(1.5));
                 if_signed_else_empty_str! {
                     $Signedness;
                     "assert_eq!(Fix::from_num(-7.5).saturating_rem_euclid_int(2), Fix::from_num(0.5));
-// −8 ≤ Fix < 8, so the answer 12.5 saturates
+// -8 ≤ Fix < 8, so the answer 12.5 saturates
 assert_eq!(Fix::from_num(-7.5).saturating_rem_euclid_int(20), Fix::MAX);
 ",
                 },
@@ -1140,6 +1234,59 @@ assert_eq!(Fix::from_num(-7.5).saturating_rem_euclid_int(20), Fix::MAX);
                 }
             }
 
+            comment! {
+                "Linear interpolation between `start` and `end`, saturating on
+overflow.
+
+The interpolated value is
+`start`&nbsp;+&nbsp;`self`&nbsp;×&nbsp;(`end`&nbsp;&minus;&nbsp;`start`). This
+is `start` when `self`&nbsp;=&nbsp;0, `end` when `self`&nbsp;=&nbsp;1, and
+linear interpolation for all other values of `self`. Linear extrapolation is
+performed if `self` is not in the range 0&nbsp;≤&nbsp;<i>x</i>&nbsp;≤&nbsp;1.
+
+# Examples
+
+```rust
+use fixed::{types::extra::U4, ", $s_fixed, "};
+type Fix = ", $s_fixed, "<U4>;
+assert_eq!(Fix::from_num(0.5).saturating_lerp(Fix::ZERO, Fix::MAX), Fix::MAX / 2);
+assert_eq!(Fix::from_num(1.5).saturating_lerp(Fix::ZERO, Fix::MAX), Fix::MAX);
+",
+                if_signed_unsigned! (
+                    $Signedness,
+                    "assert_eq!(Fix::from_num(-2.0).saturating_lerp(Fix::ZERO, Fix::MAX), Fix::MIN);
+assert_eq!(Fix::from_num(3.0).saturating_lerp(Fix::MAX, Fix::ZERO), Fix::MIN);
+",
+                    "assert_eq!(Fix::from_num(3.0).saturating_lerp(Fix::MAX, Fix::ZERO), Fix::ZERO);
+",
+                ),
+                "```
+";
+                #[inline]
+                pub fn saturating_lerp<RangeFrac>(
+                    self,
+                    start: $Fixed<RangeFrac>,
+                    end: $Fixed<RangeFrac>,
+                ) -> $Fixed<RangeFrac> {
+                    match lerp::$Inner(self.to_bits(), start.to_bits(), end.to_bits(), Frac::U32) {
+                        (bits, false) => $Fixed::from_bits(bits),
+                        (_, true) => if_signed_unsigned!(
+                            $Signedness,
+                            if (self < 0) == (end.to_bits() < start.to_bits()) {
+                                $Fixed::MAX
+                            } else {
+                                $Fixed::MIN
+                            },
+                            if end.to_bits() < start.to_bits() {
+                                $Fixed::MIN
+                            } else {
+                                $Fixed::MAX
+                            },
+                        ),
+                    }
+                }
+            }
+
             if_signed! {
                 $Signedness;
                 comment! {
@@ -1150,7 +1297,7 @@ Overflow can only occur
   * if the value is positive and the fixed-point number has zero
     or one integer bits such that it cannot hold the value 1.
   * if the value is negative and the fixed-point number has zero
-    integer bits, such that it cannot hold the value −1.
+    integer bits, such that it cannot hold the value &minus;1.
 
 # Examples
 
@@ -1172,6 +1319,7 @@ assert_eq!(ZeroIntBits::from_num(-0.5).wrapping_signum(), 0);
 ```
 ";
                     #[inline]
+                    #[must_use]
                     pub fn wrapping_signum(self) -> $Fixed<Frac> {
                         match self.to_bits().cmp(&0) {
                             Ordering::Equal => Self::ZERO,
@@ -1198,7 +1346,8 @@ assert_eq!(Fix::MAX.wrapping_mul(Fix::from_num(4)), wrapped);
                 #[inline]
                 #[must_use = "this returns the result of the operation, without modifying the original"]
                 pub fn wrapping_mul(self, rhs: $Fixed<Frac>) -> $Fixed<Frac> {
-                    let (ans, _) = self.to_bits().mul_overflow(rhs.to_bits(), Frac::U32);
+                    let (ans, _) =
+                        arith::overflowing_mul(self.to_bits(), rhs.to_bits(), Frac::U32);
                     Self::from_bits(ans)
                 }
             }
@@ -1225,7 +1374,8 @@ assert_eq!(Fix::MAX.wrapping_div(quarter), wrapped);
                 #[inline]
                 #[must_use = "this returns the result of the operation, without modifying the original"]
                 pub fn wrapping_div(self, rhs: $Fixed<Frac>) -> $Fixed<Frac> {
-                    let (ans, _) = self.to_bits().div_overflow(rhs.to_bits(), Frac::U32);
+                    let (ans, _) =
+                        arith::overflowing_div(self.to_bits(), rhs.to_bits(), Frac::U32);
                     Self::from_bits(ans)
                 }
             }
@@ -1248,6 +1398,7 @@ assert_eq!(Fix::from_num(0.25).wrapping_recip(), Fix::ZERO);
 ```
 ";
                 #[inline]
+                #[must_use]
                 pub fn wrapping_recip(self) -> $Fixed<Frac> {
                     let (ans, _) = self.overflowing_recip();
                     ans
@@ -1284,7 +1435,7 @@ assert_eq!(Fix::MAX.wrapping_div_euclid(Fix::from_num(0.25)), wrapped);
                     $Signedness,
                     ", wrapping on overflow.
 
-Overflow can only occur when dividing the minimum value by −1.",
+Overflow can only occur when dividing the minimum value by &minus;1.",
                     ".
 
 Can never overflow for unsigned values.",
@@ -1345,7 +1496,8 @@ assert_eq!(acc, Fix::MAX.wrapping_mul_int(4));
                     a: $Fixed<AFrac>,
                     b: $Fixed<BFrac>,
                 ) {
-                    let (ans, _) = a.to_bits().mul_add_overflow(
+                    let (ans, _) = arith::overflowing_mul_add(
+                        a.to_bits(),
                         b.to_bits(),
                         self.to_bits(),
                         AFrac::I32 + BFrac::I32 - Frac::I32,
@@ -1382,7 +1534,7 @@ assert_eq!(Fix::from_num(7.5).wrapping_rem_euclid_int(2), Fix::from_num(1.5));
                 if_signed_else_empty_str! {
                     $Signedness;
                     "assert_eq!(Fix::from_num(-7.5).wrapping_rem_euclid_int(2), Fix::from_num(0.5));
-// −8 ≤ Fix < 8, so the answer 12.5 wraps to −3.5
+// -8 ≤ Fix < 8, so the answer 12.5 wraps to -3.5
 assert_eq!(Fix::from_num(-7.5).wrapping_rem_euclid_int(20), Fix::from_num(-3.5));
 ",
                 },
@@ -1395,6 +1547,40 @@ assert_eq!(Fix::from_num(-7.5).wrapping_rem_euclid_int(20), Fix::from_num(-3.5))
                 }
             }
 
+            comment! {
+                "Linear interpolation between `start` and `end`, wrapping on
+overflow.
+
+The interpolated value is
+`start`&nbsp;+&nbsp;`self`&nbsp;×&nbsp;(`end`&nbsp;&minus;&nbsp;`start`). This
+is `start` when `self`&nbsp;=&nbsp;0, `end` when `self`&nbsp;=&nbsp;1, and
+linear interpolation for all other values of `self`. Linear extrapolation is
+performed if `self` is not in the range 0&nbsp;≤&nbsp;<i>x</i>&nbsp;≤&nbsp;1.
+
+# Examples
+
+```rust
+use fixed::{types::extra::U4, ", $s_fixed, "};
+type Fix = ", $s_fixed, "<U4>;
+assert_eq!(Fix::from_num(0.5).wrapping_lerp(Fix::ZERO, Fix::MAX), Fix::MAX / 2);
+assert_eq!(
+    Fix::from_num(1.5).wrapping_lerp(Fix::ZERO, Fix::MAX),
+    Fix::MAX.wrapping_add(Fix::MAX / 2)
+);
+```
+";
+                #[inline]
+                pub fn wrapping_lerp<RangeFrac>(
+                    self,
+                    start: $Fixed<RangeFrac>,
+                    end: $Fixed<RangeFrac>,
+                ) -> $Fixed<RangeFrac> {
+                    let (bits, _) =
+                        lerp::$Inner(self.to_bits(), start.to_bits(), end.to_bits(), Frac::U32);
+                    $Fixed::from_bits(bits)
+                }
+            }
+
             if_signed! {
                 $Signedness;
                 comment! {
@@ -1405,7 +1591,7 @@ Overflow can only occur
   * if the value is positive and the fixed-point number has zero
     or one integer bits such that it cannot hold the value 1.
   * if the value is negative and the fixed-point number has zero
-    integer bits, such that it cannot hold the value −1.
+    integer bits, such that it cannot hold the value &minus;1.
 
 # Panics
 
@@ -1431,6 +1617,7 @@ let _overflow = OneIntBit::from_num(0.5).unwrapped_signum();
 ";
                     #[inline]
                     #[track_caller]
+                    #[must_use]
                     pub fn unwrapped_signum(self) -> $Fixed<Frac> {
                         self.checked_signum().expect("overflow")
                     }
@@ -1522,6 +1709,7 @@ assert_eq!(Fix::from_num(0.25).unwrapped_recip(), Fix::from_num(4));
 ";
                 #[inline]
                 #[track_caller]
+                #[must_use]
                 pub fn unwrapped_recip(self) -> $Fixed<Frac> {
                     match self.overflowing_recip() {
                         (_, true) => panic!("overflow"),
@@ -1596,7 +1784,7 @@ assert_eq!(acc, 5);
                 if_signed_else_empty_str! {
                     $Signedness;
                     "
-// MAX × 1.5 − MAX = MAX / 2, which does not overflow
+// MAX × 1.5 - MAX = MAX / 2, which does not overflow
 acc = -Fix::MAX;
 acc.unwrapped_mul_acc(Fix::MAX, Fix::from_num(1.5));
 assert_eq!(acc, Fix::MAX / 2);
@@ -1620,14 +1808,13 @@ acc.unwrapped_mul_acc(Fix::MAX, Fix::ONE);
                     a: $Fixed<AFrac>,
                     b: $Fixed<BFrac>,
                 ) {
-                    let (ans, overflow) = a.to_bits().mul_add_overflow(
+                    let (ans, overflow) = arith::overflowing_mul_add(
+                        a.to_bits(),
                         b.to_bits(),
                         self.to_bits(),
                         AFrac::I32 + BFrac::I32 - Frac::I32,
                     );
-                    if overflow {
-                        panic!("overflow");
-                    }
+                    assert!(!overflow, "overflow");
                     *self = Self::from_bits(ans);
                 }
             }
@@ -1670,7 +1857,7 @@ let _divisor_is_zero = Fix::from_num(3.75).unwrapped_rem_int(0);
                     $Signedness,
                     ", panicking on overflow.
 
-Overflow can only occur when dividing the minimum value by −1.",
+Overflow can only occur when dividing the minimum value by &minus;1.",
                     ".
 
 Can never overflow for unsigned values.",
@@ -1759,7 +1946,7 @@ The following panics because of overflow.
 ```should_panic
 use fixed::{types::extra::U", $s_nbits_m4, ", ", $s_fixed, "};
 type Fix = ", $s_fixed, "<U", $s_nbits_m4, ">;
-// −8 ≤ Fix < 8, so the answer 12.5 overflows
+// -8 ≤ Fix < 8, so the answer 12.5 overflows
 let _overflow = Fix::from_num(-7.5).unwrapped_rem_euclid_int(20);
 ",
                 },
@@ -1776,6 +1963,49 @@ let _overflow = Fix::from_num(-7.5).unwrapped_rem_euclid_int(20);
                 }
             }
 
+            comment! {
+                "Linear interpolation between `start` and `end`, panicking on
+overflow.
+
+The interpolated value is
+`start`&nbsp;+&nbsp;`self`&nbsp;×&nbsp;(`end`&nbsp;&minus;&nbsp;`start`). This
+is `start` when `self`&nbsp;=&nbsp;0, `end` when `self`&nbsp;=&nbsp;1, and
+linear interpolation for all other values of `self`. Linear extrapolation is
+performed if `self` is not in the range 0&nbsp;≤&nbsp;<i>x</i>&nbsp;≤&nbsp;1.
+
+# Panics
+
+Panics if the result overflows.
+
+# Examples
+
+```rust
+use fixed::{types::extra::U4, ", $s_fixed, "};
+type Fix = ", $s_fixed, "<U4>;
+assert_eq!(Fix::from_num(0.5).unwrapped_lerp(Fix::ZERO, Fix::MAX), Fix::MAX / 2);
+```
+
+The following panics because of overflow.
+
+```should_panic
+use fixed::{types::extra::U4, ", $s_fixed, "};
+type Fix = ", $s_fixed, "<U4>;
+let _overflow = Fix::from_num(1.5).unwrapped_lerp(Fix::ZERO, Fix::MAX);
+```
+";
+                #[inline]
+                pub fn unwrapped_lerp<RangeFrac>(
+                    self,
+                    start: $Fixed<RangeFrac>,
+                    end: $Fixed<RangeFrac>,
+                ) -> $Fixed<RangeFrac> {
+                    match lerp::$Inner(self.to_bits(), start.to_bits(), end.to_bits(), Frac::U32) {
+                        (bits, false) => $Fixed::from_bits(bits),
+                        (_, true) => panic!("overflow"),
+                    }
+                }
+            }
+
             if_signed! {
                 $Signedness;
                 comment! {
@@ -1788,7 +2018,7 @@ Overflow can only occur
   * if the value is positive and the fixed-point number has zero
     or one integer bits such that it cannot hold the value 1.
   * if the value is negative and the fixed-point number has zero
-    integer bits, such that it cannot hold the value −1.
+    integer bits, such that it cannot hold the value &minus;1.
 
 # Examples
 
@@ -1839,7 +2069,8 @@ assert_eq!(Fix::MAX.overflowing_mul(Fix::from_num(4)), (wrapped, true));
                 #[inline]
                 #[must_use = "this returns the result of the operation, without modifying the original"]
                 pub fn overflowing_mul(self, rhs: $Fixed<Frac>) -> ($Fixed<Frac>, bool) {
-                    let (ans, overflow) = self.to_bits().mul_overflow(rhs.to_bits(), Frac::U32);
+                    let (ans, overflow) =
+                        arith::overflowing_mul(self.to_bits(), rhs.to_bits(), Frac::U32);
                     (Self::from_bits(ans), overflow)
                 }
             }
@@ -1869,13 +2100,14 @@ assert_eq!(Fix::MAX.overflowing_div(quarter), (wrapped, true));
                 #[inline]
                 #[must_use = "this returns the result of the operation, without modifying the original"]
                 pub fn overflowing_div(self, rhs: $Fixed<Frac>) -> ($Fixed<Frac>, bool) {
-                    let (ans, overflow) = self.to_bits().div_overflow(rhs.to_bits(), Frac::U32);
+                    let (ans, overflow) =
+                        arith::overflowing_div(self.to_bits(), rhs.to_bits(), Frac::U32);
                     (Self::from_bits(ans), overflow)
                 }
             }
 
             comment! {
-                "Overflowing reciprocal. 
+                "Overflowing reciprocal.
 
 Returns a [tuple] of the reciprocal and a [`bool`] indicating whether
 an overflow has occurred. On overflow, the wrapped value is returned.
@@ -1905,8 +2137,7 @@ assert_eq!(Small::from_num(0.25).overflowing_recip(), (Small::ZERO, true));
                     }
                     if_signed! {
                         $Signedness;
-                        use crate::int_helper::IntHelper;
-                        let (neg, abs) = self.to_bits().neg_abs();
+                        let (neg, abs) = int_helper::$Inner::neg_abs(self.to_bits());
                         let uns_abs = $UFixed::<Frac>::from_bits(abs);
                         let (uns_wrapped, overflow1) = uns_abs.overflowing_recip();
                         let (wrapped, overflow2) =
@@ -1934,7 +2165,7 @@ assert_eq!(Small::from_num(0.25).overflowing_recip(), (Small::ZERO, true));
             }
 
             comment! {
-                "Overflowing Euclidean division. 
+                "Overflowing Euclidean division.
 
 Returns a [tuple] of the quotient and a [`bool`] indicating whether an
 overflow has occurred. On overflow, the wrapped value is returned.
@@ -1990,7 +2221,7 @@ Returns a [tuple] of the quotient and ",
                     $Signedness,
                     "a [`bool`] indicating whether an overflow has
 occurred. On overflow, the wrapped value is returned. Overflow can
-only occur when dividing the minimum value by −1.",
+only occur when dividing the minimum value by &minus;1.",
                     "[`false`], as the division can never overflow for unsigned values.",
                 ),
                 "
@@ -2075,7 +2306,7 @@ assert_eq!(acc, Fix::MAX.wrapping_mul_int(4));
                 if_signed_else_empty_str! {
                     $Signedness;
                     "
-// MAX × 1.5 − MAX = MAX / 2, which does not overflow
+// MAX × 1.5 - MAX = MAX / 2, which does not overflow
 acc = -Fix::MAX;
 assert!(!acc.overflowing_mul_acc(Fix::MAX, Fix::from_num(1.5)));
 assert_eq!(acc, Fix::MAX / 2);
@@ -2090,7 +2321,8 @@ assert_eq!(acc, Fix::MAX / 2);
                     a: $Fixed<AFrac>,
                     b: $Fixed<BFrac>,
                 ) -> bool {
-                    let (ans, overflow) = a.to_bits().mul_add_overflow(
+                    let (ans, overflow) = arith::overflowing_mul_add(
+                        a.to_bits(),
                         b.to_bits(),
                         self.to_bits(),
                         AFrac::I32 + BFrac::I32 - Frac::I32,
@@ -2129,7 +2361,7 @@ assert_eq!(Fix::from_num(7.5).overflowing_rem_euclid_int(2), (Fix::from_num(1.5)
                 if_signed_else_empty_str! {
                     $Signedness;
                     "assert_eq!(Fix::from_num(-7.5).overflowing_rem_euclid_int(2), (Fix::from_num(0.5), false));
-// −8 ≤ Fix < 8, so the answer 12.5 wraps to −3.5
+// -8 ≤ Fix < 8, so the answer 12.5 wraps to -3.5
 assert_eq!(Fix::from_num(-7.5).overflowing_rem_euclid_int(20), (Fix::from_num(-3.5), true));
 ",
                 },
@@ -2167,6 +2399,45 @@ assert_eq!(Fix::from_num(-7.5).overflowing_rem_euclid_int(20), (Fix::from_num(-3
                     }
                 }
             }
+
+            comment! {
+                "Overflowing linear interpolation between `start` and `end`.
+
+Returns a [tuple] of the result and a [`bool`] indicationg whether an overflow
+has occurred. On overflow, the wrapped value is returned.
+
+The interpolated value is
+`start`&nbsp;+&nbsp;`self`&nbsp;×&nbsp;(`end`&nbsp;&minus;&nbsp;`start`). This
+is `start` when `self`&nbsp;=&nbsp;0, `end` when `self`&nbsp;=&nbsp;1, and
+linear interpolation for all other values of `self`. Linear extrapolation is
+performed if `self` is not in the range 0&nbsp;≤&nbsp;<i>x</i>&nbsp;≤&nbsp;1.
+
+# Examples
+
+```rust
+use fixed::{types::extra::U4, ", $s_fixed, "};
+type Fix = ", $s_fixed, "<U4>;
+assert_eq!(
+    Fix::from_num(0.5).overflowing_lerp(Fix::ZERO, Fix::MAX), 
+    (Fix::MAX / 2, false)
+);
+assert_eq!(
+    Fix::from_num(1.5).overflowing_lerp(Fix::ZERO, Fix::MAX),
+    (Fix::MAX.wrapping_add(Fix::MAX / 2), true)
+);
+```
+";
+                #[inline]
+                pub fn overflowing_lerp<RangeFrac>(
+                    self,
+                    start: $Fixed<RangeFrac>,
+                    end: $Fixed<RangeFrac>,
+                ) -> ($Fixed<RangeFrac>, bool) {
+                    match lerp::$Inner(self.to_bits(), start.to_bits(), end.to_bits(), Frac::U32) {
+                        (bits, overflow) => ($Fixed::from_bits(bits), overflow),
+                    }
+                }
+            }
         }
     };
 }
