@@ -1,4 +1,4 @@
-// Copyright © 2018–2021 Trevor Spiteri
+// Copyright © 2018–2022 Trevor Spiteri
 
 // This library is free software: you can redistribute it and/or
 // modify it under the terms of either
@@ -24,7 +24,7 @@ macro_rules! fixed_round {
                 concat!(
                     "Note that since the numbers are stored in two’s
 complement, negative numbers with non-zero fractional parts will be
-rounded towards −∞, except in the case where there are no integer
+rounded towards &minus;∞, except in the case where there are no integer
 bits, that is `", $s_fixed, "<U", $s_nbits, ">`, where the return value is always zero.",
                 ),
                 "Note that for unsigned numbers, this is equivalent to [`floor`].",
@@ -59,6 +59,7 @@ assert_eq!((-two_and_quarter).int(), -three);
 "
             };
             #[inline]
+            #[must_use]
             pub fn int(self) -> $Fixed<Frac> {
                 Self::from_bits(self.to_bits() & Self::INT_MASK)
             }
@@ -100,6 +101,7 @@ assert_eq!((-two_and_quarter).frac(), three_quarters);
             "```
 ";
             #[inline]
+            #[must_use]
             pub fn frac(self) -> $Fixed<Frac> {
                 Self::from_bits(self.to_bits() & Self::FRAC_MASK)
             }
@@ -115,7 +117,7 @@ assert_eq!((-two_and_quarter).frac(), three_quarters);
 from truncating/discarding the fractional bits. This is because in
 two’s-complement representations, the value of all the bits except for
 the most significant bit is positive; discarding positive bits would
-round towards −∞ unlike this method which rounds towards zero.",
+round towards &minus;∞ unlike this method which rounds towards zero.",
                 "Note that for unsigned numbers, this is equivalent to [`floor`].",
             ),
             "
@@ -143,6 +145,7 @@ assert_eq!(Fix::from_num(-2.9).round_to_zero(), Fix::from_num(-2));
 "
             };
             #[inline]
+            #[must_use]
             pub fn round_to_zero(self) -> $Fixed<Frac> {
                 if_signed! {
                     $Signedness;
@@ -187,6 +190,7 @@ assert_eq!(Fix::from_num(2.5).ceil(), Fix::from_num(3));
 [`wrapping_ceil`]: Self::wrapping_ceil
 ";
             #[inline]
+            #[must_use]
             pub fn ceil(self) -> $Fixed<Frac> {
                 let (ceil, overflow) = self.overflowing_ceil();
                 debug_assert!(!overflow, "overflow");
@@ -196,7 +200,7 @@ assert_eq!(Fix::from_num(2.5).ceil(), Fix::from_num(3));
         }
 
         comment! {
-            "Rounds to the next integer towards −∞.
+            "Rounds to the next integer towards &minus;∞.
 
 ",
             if_signed_else_empty_str! {
@@ -229,6 +233,7 @@ assert_eq!(Fix::from_num(2.5).floor(), Fix::from_num(2));
 [`wrapping_floor`]: Self::wrapping_floor
 ";
             #[inline]
+            #[must_use]
             pub fn floor(self) -> $Fixed<Frac> {
                 let (floor, overflow) = self.overflowing_floor();
                 debug_assert!(!overflow, "overflow");
@@ -265,6 +270,7 @@ assert_eq!(Fix::from_num(2.5).round(), Fix::from_num(3));
 [`wrapping_round`]: Self::wrapping_round
 ";
             #[inline]
+            #[must_use]
             pub fn round(self) -> $Fixed<Frac> {
                 let (round, overflow) = self.overflowing_round();
                 debug_assert!(!overflow, "overflow");
@@ -296,6 +302,7 @@ assert_eq!(Fix::from_num(3.5).round_ties_to_even(), Fix::from_num(4));
 [`wrapping_round_ties_to_even`]: Self::wrapping_round_ties_to_even
 ";
             #[inline]
+            #[must_use]
             pub fn round_ties_to_even(self) -> $Fixed<Frac> {
                 let (round, overflow) = self.overflowing_round_ties_to_even();
                 debug_assert!(!overflow, "overflow");
@@ -331,7 +338,7 @@ assert_eq!(Fix::from_num(2.5).checked_ceil(), Some(Fix::from_num(3)));
         }
 
         comment! {
-            "Checked floor. Rounds to the next integer towards −∞.",
+            "Checked floor. Rounds to the next integer towards &minus;∞.",
             if_signed_unsigned!(
                 $Signedness,
                 "Returns [`None`] on overflow.
@@ -442,6 +449,7 @@ assert_eq!(Fix::from_num(2.5).saturating_ceil(), Fix::from_num(3));
 ```
 ";
             #[inline]
+            #[must_use]
             pub fn saturating_ceil(self) -> $Fixed<Frac> {
                 let (ceil, overflow) = self.overflowing_ceil();
                 if overflow { Self::MAX } else { ceil }
@@ -449,7 +457,7 @@ assert_eq!(Fix::from_num(2.5).saturating_ceil(), Fix::from_num(3));
         }
 
         comment! {
-            "Saturating floor. Rounds to the next integer towards −∞",
+            "Saturating floor. Rounds to the next integer towards &minus;∞",
             if_signed_unsigned!(
                 $Signedness,
                 ", saturating on overflow.
@@ -487,6 +495,7 @@ assert_eq!(AllFrac::MIN.saturating_floor(), AllFrac::MIN);
             "```
 ";
             #[inline]
+            #[must_use]
             pub fn saturating_floor(self) -> $Fixed<Frac> {
                 let (floor, overflow) = self.overflowing_floor();
                 if overflow { Self::MIN } else { floor }
@@ -513,6 +522,7 @@ assert_eq!(Fix::from_num(2.5).saturating_round(), Fix::from_num(3));
 ```
 ";
             #[inline]
+            #[must_use]
             pub fn saturating_round(self) -> $Fixed<Frac> {
                 let saturated = if self.to_bits() > 0 {
                     $Fixed::MAX
@@ -539,6 +549,7 @@ assert_eq!(Fix::MAX.saturating_round_ties_to_even(), Fix::MAX);
 ```
 ";
             #[inline]
+            #[must_use]
             pub fn saturating_round_ties_to_even(self) -> $Fixed<Frac> {
                 let saturated = if self.to_bits() > 0 {
                     $Fixed::MAX
@@ -570,13 +581,14 @@ assert_eq!(Fix::from_num(2.5).wrapping_ceil(), Fix::from_num(3));
 ```
 ";
             #[inline]
+            #[must_use]
             pub fn wrapping_ceil(self) -> $Fixed<Frac> {
                 self.overflowing_ceil().0
             }
         }
 
         comment! {
-            "Wrapping floor. Rounds to the next integer towards −∞",
+            "Wrapping floor. Rounds to the next integer towards &minus;∞",
             if_signed_unsigned!(
                 $Signedness,
                 ", wrapping on overflow.
@@ -614,6 +626,7 @@ assert_eq!(AllFrac::MIN.wrapping_floor(), AllFrac::ZERO);
             "```
 ";
             #[inline]
+            #[must_use]
             pub fn wrapping_floor(self) -> $Fixed<Frac> {
                 self.overflowing_floor().0
             }
@@ -639,6 +652,7 @@ assert_eq!(Fix::from_num(2.5).wrapping_round(), Fix::from_num(3));
 ```
 ";
             #[inline]
+            #[must_use]
             pub fn wrapping_round(self) -> $Fixed<Frac> {
                 self.overflowing_round().0
             }
@@ -659,6 +673,7 @@ assert_eq!(Fix::MAX.wrapping_round_ties_to_even(), Fix::MIN);
 ```
 ";
             #[inline]
+            #[must_use]
             pub fn wrapping_round_ties_to_even(self) -> $Fixed<Frac> {
                 self.overflowing_round_ties_to_even().0
             }
@@ -696,13 +711,14 @@ let _overflow = Fix::MAX.unwrapped_ceil();
 ";
             #[inline]
             #[track_caller]
+            #[must_use]
             pub fn unwrapped_ceil(self) -> $Fixed<Frac> {
                 self.checked_ceil().expect("overflow")
             }
         }
 
         comment! {
-            "Unwrapped floor. Rounds to the next integer towards −∞",
+            "Unwrapped floor. Rounds to the next integer towards &minus;∞",
             if_signed_unsigned!(
                 $Signedness,
                 ", panicking on overflow.
@@ -740,6 +756,7 @@ let _overflow = AllFrac::MIN.unwrapped_floor();
 ";
             #[inline]
             #[track_caller]
+            #[must_use]
             pub fn unwrapped_floor(self) -> $Fixed<Frac> {
                 self.checked_floor().expect("overflow")
             }
@@ -777,6 +794,7 @@ let _overflow = Fix::MAX.unwrapped_round();
 ";
             #[inline]
             #[track_caller]
+            #[must_use]
             pub fn unwrapped_round(self) -> $Fixed<Frac> {
                 self.checked_round().expect("overflow")
             }
@@ -809,6 +827,7 @@ let _overflow = Fix::MAX.unwrapped_round_ties_to_even();
 ";
             #[inline]
             #[track_caller]
+            #[must_use]
             pub fn unwrapped_round_ties_to_even(self) -> $Fixed<Frac> {
                 self.checked_round_ties_to_even().expect("overflow")
             }
@@ -858,7 +877,7 @@ assert_eq!(Fix::from_num(2.5).overflowing_ceil(), (Fix::from_num(3), false));
         }
 
         comment! {
-            "Overflowing floor. Rounds to the next integer towards −∞.
+            "Overflowing floor. Rounds to the next integer towards &minus;∞.
 
 Returns a [tuple] of the fixed-point number and
 ",
@@ -949,8 +968,8 @@ assert_eq!(Fix::from_num(2.5).overflowing_round(), (Fix::from_num(3), false));
                         // otherwise .100...01, 0 < x < -0.5,  no overflow
                         return (int, tie);
                     }
-                    // If num is −int.100...00 = (-int) + 0.5, we simply truncate to move to −∞.
-                    // If num is −int.100...01 = (-int) + 0.6, we add 1 to −int.
+                    // If num is -int.100...00 = (-int) + 0.5, we simply truncate to move to -∞.
+                    // If num is -int.100...01 = (-int) + 0.6, we add 1 to -int.
                     // If num is +int.100...00 = (+int) + 0.5, we add 1 to +int.
                     // If num is +int.100...01 = (+int) + 0.6, we add 1 to +int.
                     if tie && self.to_bits() < 0 {
