@@ -1,4 +1,4 @@
-// Copyright © 2018–2021 Trevor Spiteri
+// Copyright © 2018–2022 Trevor Spiteri
 
 // This library is free software: you can redistribute it and/or modify it under
 // the terms of either
@@ -14,7 +14,9 @@
 // <https://opensource.org/licenses/MIT>.
 
 use crate::{
-    helpers::{FloatHelper, FloatKind, FromFloatHelper, IntHelper},
+    float_helper,
+    helpers::{FloatKind, FromFloatHelper},
+    int_helper::IntFixed,
     traits::{Fixed, FixedEquiv, FromFixed, ToFixed},
     types::extra::U0,
     F128Bits, FixedI128, FixedI16, FixedI32, FixedI64, FixedI8, FixedU128, FixedU16, FixedU32,
@@ -87,7 +89,7 @@ macro_rules! impl_int {
         impl FromFixed for $Int {
             /// Converts a fixed-point number to an integer.
             ///
-            /// Any fractional bits are discarded, which rounds towards −∞.
+            /// Any fractional bits are discarded, which rounds towards &minus;∞.
             ///
             /// # Panics
             ///
@@ -101,31 +103,31 @@ macro_rules! impl_int {
             /// [`wrapping_from_fixed`]: FromFixed::wrapping_from_fixed
             #[inline]
             fn from_fixed<F: Fixed>(src: F) -> Self {
-                $Int::from_repr_fixed(FromFixed::from_fixed(src))
+                IntFixed::<$Int>::int(FromFixed::from_fixed(src))
             }
 
             /// Converts a fixed-point number to an integer if it fits, otherwise returns [`None`].
             ///
-            /// Any fractional bits are discarded, which rounds towards −∞.
+            /// Any fractional bits are discarded, which rounds towards &minus;∞.
             #[inline]
             fn checked_from_fixed<F: Fixed>(src: F) -> Option<Self> {
-                FromFixed::checked_from_fixed(src).map($Int::from_repr_fixed)
+                FromFixed::checked_from_fixed(src).map(IntFixed::<$Int>::int)
             }
 
             /// Converts a fixed-point number to an integer, saturating if it does not fit.
             ///
-            /// Any fractional bits are discarded, which rounds towards −∞.
+            /// Any fractional bits are discarded, which rounds towards &minus;∞.
             #[inline]
             fn saturating_from_fixed<F: Fixed>(src: F) -> Self {
-                $Int::from_repr_fixed(FromFixed::saturating_from_fixed(src))
+                IntFixed::<$Int>::int(FromFixed::saturating_from_fixed(src))
             }
 
             /// Converts a fixed-point number to an integer, wrapping if it does not fit.
             ///
-            /// Any fractional bits are discarded, which rounds towards −∞.
+            /// Any fractional bits are discarded, which rounds towards &minus;∞.
             #[inline]
             fn wrapping_from_fixed<F: Fixed>(src: F) -> Self {
-                $Int::from_repr_fixed(FromFixed::wrapping_from_fixed(src))
+                IntFixed::<$Int>::int(FromFixed::wrapping_from_fixed(src))
             }
 
             /// Converts a fixed-point number to an integer.
@@ -134,16 +136,16 @@ macro_rules! impl_int {
             /// an overflow has occurred. On overflow, the wrapped value is
             /// returned.
             ///
-            /// Any fractional bits are discarded, which rounds towards −∞.
+            /// Any fractional bits are discarded, which rounds towards &minus;∞.
             #[inline]
             fn overflowing_from_fixed<F: Fixed>(src: F) -> (Self, bool) {
-                let (repr_fixed, overflow) = FromFixed::overflowing_from_fixed(src);
-                ($Int::from_repr_fixed(repr_fixed), overflow)
+                let (fixed, overflow) = FromFixed::overflowing_from_fixed(src);
+                (IntFixed::<$Int>::int(fixed), overflow)
             }
 
             /// Converts a fixed-point number to an integer, panicking if it does not fit.
             ///
-            /// Any fractional bits are discarded, which rounds towards −∞.
+            /// Any fractional bits are discarded, which rounds towards &minus;∞.
             ///
             /// # Panics
             ///
@@ -151,7 +153,7 @@ macro_rules! impl_int {
             /// does not fit, even when debug assertions are not enabled.
             #[inline]
             fn unwrapped_from_fixed<F: Fixed>(src: F) -> Self {
-                $Int::from_repr_fixed(FromFixed::unwrapped_from_fixed(src))
+                IntFixed::<$Int>::int(FromFixed::unwrapped_from_fixed(src))
             }
         }
 
@@ -170,25 +172,25 @@ macro_rules! impl_int {
             /// [`wrapping_to_fixed`]: ToFixed::wrapping_to_fixed
             #[inline]
             fn to_fixed<F: Fixed>(self) -> F {
-                ToFixed::to_fixed(self.to_repr_fixed())
+                ToFixed::to_fixed(IntFixed(self).fixed())
             }
 
             /// Converts an integer to a fixed-point number if it fits, otherwise returns [`None`].
             #[inline]
             fn checked_to_fixed<F: Fixed>(self) -> Option<F> {
-                ToFixed::checked_to_fixed(self.to_repr_fixed())
+                ToFixed::checked_to_fixed(IntFixed(self).fixed())
             }
 
             /// Converts an integer to a fixed-point number, saturating if it does not fit.
             #[inline]
             fn saturating_to_fixed<F: Fixed>(self) -> F {
-                ToFixed::saturating_to_fixed(self.to_repr_fixed())
+                ToFixed::saturating_to_fixed(IntFixed(self).fixed())
             }
 
             /// Converts an integer to a fixed-point number, wrapping if it does not fit.
             #[inline]
             fn wrapping_to_fixed<F: Fixed>(self) -> F {
-                ToFixed::wrapping_to_fixed(self.to_repr_fixed())
+                ToFixed::wrapping_to_fixed(IntFixed(self).fixed())
             }
 
             /// Converts an integer to a fixed-point number.
@@ -198,7 +200,7 @@ macro_rules! impl_int {
             /// wrapped value is returned.
             #[inline]
             fn overflowing_to_fixed<F: Fixed>(self) -> (F, bool) {
-                ToFixed::overflowing_to_fixed(self.to_repr_fixed())
+                ToFixed::overflowing_to_fixed(IntFixed(self).fixed())
             }
 
             /// Converts an integer to a fixed-point number, panicking if it does not fit.
@@ -209,7 +211,7 @@ macro_rules! impl_int {
             /// assertions are not enabled.
             #[inline]
             fn unwrapped_to_fixed<F: Fixed>(self) -> F {
-                ToFixed::unwrapped_to_fixed(self.to_repr_fixed())
+                ToFixed::unwrapped_to_fixed(IntFixed(self).fixed())
             }
         }
 
@@ -265,7 +267,7 @@ impl_int! { u128, FixedU128 }
 impl_int! { usize }
 
 macro_rules! impl_float {
-    ($Float:ty, $link:expr, $overflows_fmt:expr, $overflows_filt:expr) => {
+    ($Float:ident, $link:expr, $overflows_fmt:expr, $overflows_filt:expr) => {
         impl FromFixed for $Float {
             /// Converts a fixed-point number to a floating-point number.
             ///
@@ -284,7 +286,7 @@ macro_rules! impl_float {
             #[inline]
             fn from_fixed<F: Fixed>(src: F) -> Self {
                 let helper = src.private_to_float_helper();
-                FloatHelper::from_to_float_helper(helper, F::FRAC_NBITS, F::INT_NBITS)
+                float_helper::$Float::from_to_float_helper(helper, F::FRAC_NBITS, F::INT_NBITS)
             }
 
             /// Converts a fixed-point number to a floating-point
@@ -374,7 +376,7 @@ it panics; if wrapping is required use [`wrapping_to_fixed`] instead.
             /// Rounding is to the nearest, with ties rounded to even.
             #[inline]
             fn checked_to_fixed<F: Fixed>(self) -> Option<F> {
-                let kind = self.to_float_kind(F::FRAC_NBITS, F::INT_NBITS);
+                let kind = float_helper::$Float::to_float_kind(self, F::FRAC_NBITS, F::INT_NBITS);
                 match kind {
                     FloatKind::Finite { .. } => {
                         let helper = FromFloatHelper { kind };
@@ -401,7 +403,8 @@ Panics if `self` is [NaN].
 ";
                 #[inline]
                 fn saturating_to_fixed<F: Fixed>(self) -> F {
-                    let kind = self.to_float_kind(F::FRAC_NBITS, F::INT_NBITS);
+                    let kind =
+                        float_helper::$Float::to_float_kind(self, F::FRAC_NBITS, F::INT_NBITS);
                     let helper = FromFloatHelper { kind };
                     F::private_saturating_from_float_helper(helper)
                 }
@@ -444,7 +447,8 @@ Panics if `self` is not [finite].
                 #[inline]
                 #[track_caller]
                 fn overflowing_to_fixed<F: Fixed>(self) -> (F, bool) {
-                    let kind = self.to_float_kind(F::FRAC_NBITS, F::INT_NBITS);
+                    let kind =
+                        float_helper::$Float::to_float_kind(self, F::FRAC_NBITS, F::INT_NBITS);
                     let helper = FromFloatHelper { kind };
                     F::private_overflowing_from_float_helper(helper)
                 }
