@@ -1,4 +1,4 @@
-// Copyright © 2018–2021 Trevor Spiteri
+// Copyright © 2018–2022 Trevor Spiteri
 
 // This library is free software: you can redistribute it and/or
 // modify it under the terms of either
@@ -243,6 +243,7 @@ impl<F: Fixed> Unwrapped<F> {
     /// }
     /// ```
     #[inline]
+    #[must_use]
     pub fn to_be(self) -> Self {
         Unwrapped(self.0.to_be())
     }
@@ -264,6 +265,7 @@ impl<F: Fixed> Unwrapped<F> {
     /// }
     /// ```
     #[inline]
+    #[must_use]
     pub fn to_le(self) -> Self {
         Unwrapped(self.0.to_le())
     }
@@ -282,6 +284,7 @@ impl<F: Fixed> Unwrapped<F> {
     /// assert_eq!(w.swap_bytes(), swapped);
     /// ```
     #[inline]
+    #[must_use]
     pub fn swap_bytes(self) -> Self {
         Unwrapped(self.0.swap_bytes())
     }
@@ -426,7 +429,7 @@ impl<F: Fixed> Unwrapped<F> {
     /// The other number can be:
     ///
     ///   * A fixed-point number. Any extra fractional bits are
-    ///     discarded, which rounds towards −∞.
+    ///     discarded, which rounds towards &minus;∞.
     ///   * An integer of type [`i8`], [`i16`], [`i32`], [`i64`], [`i128`],
     ///     [`isize`], [`u8`], [`u16`], [`u32`], [`u64`], [`u128`], or
     ///     [`usize`].
@@ -487,11 +490,11 @@ impl<F: Fixed> Unwrapped<F> {
     /// The other number can be:
     ///
     ///   * Another fixed-point number. Any extra fractional bits are
-    ///     discarded, which rounds towards −∞.
+    ///     discarded, which rounds towards &minus;∞.
     ///   * An integer of type [`i8`], [`i16`], [`i32`], [`i64`], [`i128`],
     ///     [`isize`], [`u8`], [`u16`], [`u32`], [`u64`], [`u128`], or
     ///     [`usize`]. Any fractional bits are discarded, which rounds
-    ///     towards −∞.
+    ///     towards &minus;∞.
     ///   * A floating-point number of type [`f16`], [`bf16`],
     ///     [`f32`], [`f64`] or [`F128Bits`]. For this conversion, the
     ///     method rounds to the nearest, with ties rounding to even.
@@ -599,7 +602,7 @@ impl<F: Fixed> Unwrapped<F> {
     ///
     /// Note that since the numbers are stored in two’s complement,
     /// negative numbers with non-zero fractional parts will be
-    /// rounded towards −∞, except in the case where there are no
+    /// rounded towards &minus;∞, except in the case where there are no
     /// integer bits, for example for the type
     /// <code>[Unwrapped]&lt;[I0F16]&gt;</code>, where the return
     /// value is always zero.
@@ -617,6 +620,7 @@ impl<F: Fixed> Unwrapped<F> {
     ///
     /// [I0F16]: crate::types::I0F16
     #[inline]
+    #[must_use]
     pub fn int(self) -> Unwrapped<F> {
         Unwrapped(self.0.int())
     }
@@ -643,6 +647,7 @@ impl<F: Fixed> Unwrapped<F> {
     ///
     /// [I0F16]: crate::types::I0F16
     #[inline]
+    #[must_use]
     pub fn frac(self) -> Unwrapped<F> {
         Unwrapped(self.0.frac())
     }
@@ -662,6 +667,7 @@ impl<F: Fixed> Unwrapped<F> {
     /// assert_eq!(Unwrapped(I16F16::from_num(-3.9)).round_to_zero(), -three);
     /// ```
     #[inline]
+    #[must_use]
     pub fn round_to_zero(self) -> Unwrapped<F> {
         Unwrapped(self.0.round_to_zero())
     }
@@ -693,11 +699,12 @@ impl<F: Fixed> Unwrapped<F> {
     /// ```
     #[inline]
     #[track_caller]
+    #[must_use]
     pub fn ceil(self) -> Unwrapped<F> {
         Unwrapped(self.0.unwrapped_ceil())
     }
 
-    /// Unwrapped floor. Rounds to the next integer towards −∞,
+    /// Unwrapped floor. Rounds to the next integer towards &minus;∞,
     /// panicking on overflow.
     ///
     /// Overflow can only occur for signed numbers with zero integer
@@ -727,6 +734,7 @@ impl<F: Fixed> Unwrapped<F> {
     /// ```
     #[inline]
     #[track_caller]
+    #[must_use]
     pub fn floor(self) -> Unwrapped<F> {
         Unwrapped(self.0.unwrapped_floor())
     }
@@ -759,6 +767,7 @@ impl<F: Fixed> Unwrapped<F> {
     /// ```
     #[inline]
     #[track_caller]
+    #[must_use]
     pub fn round(self) -> Unwrapped<F> {
         Unwrapped(self.0.unwrapped_round())
     }
@@ -794,6 +803,7 @@ impl<F: Fixed> Unwrapped<F> {
     /// ```
     #[inline]
     #[track_caller]
+    #[must_use]
     pub fn round_ties_to_even(self) -> Unwrapped<F> {
         Unwrapped(self.0.unwrapped_round_ties_to_even())
     }
@@ -910,7 +920,7 @@ impl<F: Fixed> Unwrapped<F> {
     ///
     /// # Panics
     ///
-    /// Panics if the fixed-point number is ≤ 0.
+    /// Panics if the fixed-point number is ≤&nbsp;0.
     #[inline]
     #[track_caller]
     pub fn int_log2(self) -> i32 {
@@ -924,7 +934,7 @@ impl<F: Fixed> Unwrapped<F> {
     ///
     /// # Panics
     ///
-    /// Panics if the fixed-point number is ≤ 0.
+    /// Panics if the fixed-point number is ≤&nbsp;0.
     #[inline]
     #[track_caller]
     pub fn int_log10(self) -> i32 {
@@ -1080,6 +1090,7 @@ impl<F: Fixed> Unwrapped<F> {
     /// ```
     #[inline]
     #[track_caller]
+    #[must_use]
     pub fn recip(self) -> Unwrapped<F> {
         Unwrapped(self.0.unwrapped_recip())
     }
@@ -1103,7 +1114,7 @@ impl<F: Fixed> Unwrapped<F> {
     /// let three = Unwrapped(I16F16::from_num(3));
     /// let four = Unwrapped(I16F16::from_num(4));
     /// assert_eq!(three.mul_add(half, four), Unwrapped(I16F16::from_num(5.5)));
-    /// // max × 1.5 − max = max / 2, which does not overflow
+    /// // max × 1.5 - max = max / 2, which does not overflow
     /// let max = Unwrapped(I16F16::MAX);
     /// assert_eq!(max.mul_add(Unwrapped(I16F16::from_num(1.5)), -max), max / 2);
     /// ```
@@ -1274,7 +1285,7 @@ impl<F: Fixed> Unwrapped<F> {
     /// ```should_panic
     /// use fixed::{types::I8F8, Unwrapped};
     /// let num = Unwrapped(I8F8::from_num(-7.5));
-    /// // −128 ≤ Fix < 128, so the answer 192.5 overflows
+    /// // -128 ≤ Fix < 128, so the answer 192.5 overflows
     /// let _overflow = num.rem_euclid_int(200);
     /// ```
     #[inline]
@@ -1283,6 +1294,80 @@ impl<F: Fixed> Unwrapped<F> {
     pub fn rem_euclid_int(self, divisor: F::Bits) -> Unwrapped<F> {
         Unwrapped(self.0.unwrapped_rem_euclid_int(divisor))
     }
+
+    /// Linear interpolation between `start` and `end`.
+    ///
+    /// See also
+    /// <code>FixedI32::[unwrapped\_lerp][FixedI32::unwrapped_lerp]</code> and
+    /// <code>FixedU32::[unwrapped\_lerp][FixedU32::unwrapped_lerp]</code>.
+    ///
+    /// # Panics
+    ///
+    /// Panics on overflow.
+    ///
+    /// # Examples
+    ///
+    /// ```rust
+    /// use fixed::{types::I16F16, Unwrapped};
+    /// type Unwr = Unwrapped<I16F16>;
+    /// assert_eq!(Unwr::from_num(0.5).lerp(Unwr::ZERO, Unwr::MAX), Unwr::MAX / 2);
+    /// ```
+    ///
+    /// The following panics because of overflow.
+    ///
+    /// ```should_panic
+    /// use fixed::{types::I16F16, Unwrapped};
+    /// type Unwr = Unwrapped<I16F16>;
+    /// let _overflow = Unwr::from_num(1.5).lerp(Unwr::ZERO, Unwr::MAX);
+    /// ```
+    #[inline]
+    #[must_use = "this returns the result of the operation, without modifying the original"]
+    pub fn lerp(self, start: Unwrapped<F>, end: Unwrapped<F>) -> Unwrapped<F> {
+        Unwrapped(self.0.unwrapped_lerp(start.0, end.0))
+    }
+
+    /// Inverse linear interpolation between `start` and `end`.
+    ///
+    /// See also
+    /// <code>FixedI32::[unwrapped\_inv\_lerp][FixedI32::unwrapped_inv_lerp]</code> and
+    /// <code>FixedU32::[unwrapped\_inv\_lerp][FixedU32::unwrapped_inv_lerp]</code>.
+    ///
+    /// # Panics
+    ///
+    /// Panics when `start`&nbsp;=&nbsp;`end` or when the results overflows.
+    ///
+    /// # Examples
+    ///
+    /// ```rust
+    /// use fixed::{types::I16F16, Unwrapped};
+    /// type Unwr = Unwrapped<I16F16>;
+    /// assert_eq!(
+    ///     Unwr::from_num(25).inv_lerp(Unwr::from_num(20), Unwr::from_num(40)),
+    ///     Unwr::from_num(0.25)
+    /// );
+    /// ```
+    ///
+    /// The following panics because `start`&nbsp;=&nbsp;`end`.
+    ///
+    /// ```should_panic
+    /// use fixed::{types::I16F16, Unwrapped};
+    /// type Unwr = Unwrapped<I16F16>;
+    /// let two = Unwr::from_num(2);
+    /// let _zero_range = two.inv_lerp(two, two);
+    /// ```
+    ///
+    /// The following panics because of overflow.
+    ///
+    /// ```should_panic
+    /// use fixed::{types::I16F16, Unwrapped};
+    /// type Unwr = Unwrapped<I16F16>;
+    /// let _overflow = Unwr::MAX.inv_lerp(Unwr::ZERO, Unwr::from_num(0.5));
+    /// ```
+    #[inline]
+    #[must_use]
+    pub fn inv_lerp(self, start: Unwrapped<F>, end: Unwrapped<F>) -> Unwrapped<F> {
+        Unwrapped(self.0.unwrapped_inv_lerp(start.0, end.0))
+    }
 }
 
 impl<F: FixedSigned> Unwrapped<F> {
@@ -1311,7 +1396,7 @@ impl<F: FixedSigned> Unwrapped<F> {
         self.0.signed_bits()
     }
 
-    /// Returns [`true`] if the number is > 0.
+    /// Returns [`true`] if the number is >&nbsp;0.
     ///
     /// See also <code>FixedI32::[is\_positive][FixedI32::is_positive]</code>.
     ///
@@ -1328,7 +1413,7 @@ impl<F: FixedSigned> Unwrapped<F> {
         self.0.is_positive()
     }
 
-    /// Returns [`true`] if the number is < 0.
+    /// Returns [`true`] if the number is <&nbsp;0.
     ///
     /// See also <code>FixedI32::[is\_negative][FixedI32::is_negative]</code>.
     ///
@@ -1373,6 +1458,7 @@ impl<F: FixedSigned> Unwrapped<F> {
     /// ```
     #[inline]
     #[track_caller]
+    #[must_use]
     pub fn abs(self) -> Unwrapped<F> {
         Unwrapped(self.0.unwrapped_abs())
     }
@@ -1388,7 +1474,7 @@ impl<F: FixedSigned> Unwrapped<F> {
     ///   * if the value is positive and the fixed-point number has zero
     ///     or one integer bits such that it cannot hold the value 1.
     ///   * if the value is negative and the fixed-point number has zero
-    ///     integer bits, such that it cannot hold the value −1.
+    ///     integer bits, such that it cannot hold the value &minus;1.
     ///
     /// # Examples
     ///
@@ -1407,6 +1493,7 @@ impl<F: FixedSigned> Unwrapped<F> {
     /// ```
     #[inline]
     #[track_caller]
+    #[must_use]
     pub fn signum(self) -> Unwrapped<F> {
         Unwrapped(self.0.unwrapped_signum())
     }
@@ -1454,8 +1541,8 @@ impl<F: FixedUnsigned> Unwrapped<F> {
     /// Returns the highest one in the binary representation, or zero
     /// if `self` is zero.
     ///
-    /// If `self` > 0, the highest one is equal to the largest power
-    /// of two that is ≤ `self`.
+    /// If `self`&nbsp;>&nbsp;0, the highest one is equal to the largest power
+    /// of two that is ≤&nbsp;`self`.
     ///
     /// See also <code>FixedU32::[highest\_one][FixedU32::highest_one]</code>.
     ///
@@ -1471,11 +1558,12 @@ impl<F: FixedUnsigned> Unwrapped<F> {
     /// assert_eq!(T::ZERO.highest_one(), T::ZERO);
     /// ```
     #[inline]
+    #[must_use]
     pub fn highest_one(self) -> Unwrapped<F> {
         Unwrapped(self.0.highest_one())
     }
 
-    /// Returns the smallest power of two that is ≥ `self`.
+    /// Returns the smallest power of two that is ≥&nbsp;`self`.
     ///
     /// See also
     /// <code>FixedU32::[unwrapped\_next\_power\_of\_two][FixedU32::unwrapped_next_power_of_two]</code>.
@@ -1503,6 +1591,7 @@ impl<F: FixedUnsigned> Unwrapped<F> {
     /// ```
     #[inline]
     #[track_caller]
+    #[must_use]
     pub fn next_power_of_two(self) -> Unwrapped<F> {
         Unwrapped(self.0.unwrapped_next_power_of_two())
     }
