@@ -1,4 +1,4 @@
-// Copyright © 2018–2021 Trevor Spiteri
+// Copyright © 2018–2022 Trevor Spiteri
 
 // This library is free software: you can redistribute it and/or
 // modify it under the terms of either
@@ -240,6 +240,7 @@ impl<F: Fixed> Wrapping<F> {
     /// }
     /// ```
     #[inline]
+    #[must_use]
     pub fn to_be(self) -> Self {
         Wrapping(self.0.to_be())
     }
@@ -261,6 +262,7 @@ impl<F: Fixed> Wrapping<F> {
     /// }
     /// ```
     #[inline]
+    #[must_use]
     pub fn to_le(self) -> Self {
         Wrapping(self.0.to_le())
     }
@@ -279,6 +281,7 @@ impl<F: Fixed> Wrapping<F> {
     /// assert_eq!(w.swap_bytes(), swapped);
     /// ```
     #[inline]
+    #[must_use]
     pub fn swap_bytes(self) -> Self {
         Wrapping(self.0.swap_bytes())
     }
@@ -423,7 +426,7 @@ impl<F: Fixed> Wrapping<F> {
     /// The other number can be:
     ///
     ///   * A fixed-point number. Any extra fractional bits are
-    ///     discarded, which rounds towards −∞.
+    ///     discarded, which rounds towards &minus;∞.
     ///   * An integer of type [`i8`], [`i16`], [`i32`], [`i64`], [`i128`],
     ///     [`isize`], [`u8`], [`u16`], [`u32`], [`u64`], [`u128`], or
     ///     [`usize`].
@@ -482,11 +485,11 @@ impl<F: Fixed> Wrapping<F> {
     /// The other number can be:
     ///
     ///   * Another fixed-point number. Any extra fractional bits are
-    ///     discarded, which rounds towards −∞.
+    ///     discarded, which rounds towards &minus;∞.
     ///   * An integer of type [`i8`], [`i16`], [`i32`], [`i64`], [`i128`],
     ///     [`isize`], [`u8`], [`u16`], [`u32`], [`u64`], [`u128`], or
     ///     [`usize`]. Any fractional bits are discarded, which rounds
-    ///     towards −∞.
+    ///     towards &minus;∞.
     ///   * A floating-point number of type [`f16`], [`bf16`],
     ///     [`f32`], [`f64`] or [`F128Bits`]. For this conversion, the
     ///     method rounds to the nearest, with ties rounding to even.
@@ -591,7 +594,7 @@ impl<F: Fixed> Wrapping<F> {
     ///
     /// Note that since the numbers are stored in two’s complement,
     /// negative numbers with non-zero fractional parts will be
-    /// rounded towards −∞, except in the case where there are no
+    /// rounded towards &minus;∞, except in the case where there are no
     /// integer bits, for example for the type
     /// <code>[Wrapping]&lt;[I0F16]&gt;</code>, where the return value
     /// is always zero.
@@ -609,6 +612,7 @@ impl<F: Fixed> Wrapping<F> {
     ///
     /// [I0F16]: crate::types::I0F16
     #[inline]
+    #[must_use]
     pub fn int(self) -> Wrapping<F> {
         Wrapping(self.0.int())
     }
@@ -635,6 +639,7 @@ impl<F: Fixed> Wrapping<F> {
     ///
     /// [I0F16]: crate::types::I0F16
     #[inline]
+    #[must_use]
     pub fn frac(self) -> Wrapping<F> {
         Wrapping(self.0.frac())
     }
@@ -654,6 +659,7 @@ impl<F: Fixed> Wrapping<F> {
     /// assert_eq!(Wrapping(I16F16::from_num(-3.9)).round_to_zero(), -three);
     /// ```
     #[inline]
+    #[must_use]
     pub fn round_to_zero(self) -> Wrapping<F> {
         Wrapping(self.0.round_to_zero())
     }
@@ -674,11 +680,12 @@ impl<F: Fixed> Wrapping<F> {
     /// assert_eq!(Wrapping(I16F16::MAX).ceil(), Wrapping(I16F16::MIN));
     /// ```
     #[inline]
+    #[must_use]
     pub fn ceil(self) -> Wrapping<F> {
         Wrapping(self.0.wrapping_ceil())
     }
 
-    /// Wrapping floor. Rounds to the next integer towards −∞,
+    /// Wrapping floor. Rounds to the next integer towards &minus;∞,
     /// wrapping on overflow.
     ///
     /// Overflow can only occur for signed numbers with zero integer
@@ -700,6 +707,7 @@ impl<F: Fixed> Wrapping<F> {
     /// assert_eq!(Wrapping(I0F32::MIN).floor(), Wrapping(I0F32::ZERO));
     /// ```
     #[inline]
+    #[must_use]
     pub fn floor(self) -> Wrapping<F> {
         Wrapping(self.0.wrapping_floor())
     }
@@ -721,6 +729,7 @@ impl<F: Fixed> Wrapping<F> {
     /// assert_eq!(Wrapping(I16F16::MAX).round(), Wrapping(I16F16::MIN));
     /// ```
     #[inline]
+    #[must_use]
     pub fn round(self) -> Wrapping<F> {
         Wrapping(self.0.wrapping_round())
     }
@@ -745,6 +754,7 @@ impl<F: Fixed> Wrapping<F> {
     /// assert_eq!(max.round_ties_to_even(), Wrapping(I16F16::MIN));
     /// ```
     #[inline]
+    #[must_use]
     pub fn round_ties_to_even(self) -> Wrapping<F> {
         Wrapping(self.0.wrapping_round_ties_to_even())
     }
@@ -861,7 +871,7 @@ impl<F: Fixed> Wrapping<F> {
     ///
     /// # Panics
     ///
-    /// Panics if the fixed-point number is ≤ 0.
+    /// Panics if the fixed-point number is ≤&nbsp;0.
     #[inline]
     pub fn int_log2(self) -> i32 {
         self.0.int_log2()
@@ -874,7 +884,7 @@ impl<F: Fixed> Wrapping<F> {
     ///
     /// # Panics
     ///
-    /// Panics if the fixed-point number is ≤ 0.
+    /// Panics if the fixed-point number is ≤&nbsp;0.
     #[inline]
     pub fn int_log10(self) -> i32 {
         self.0.int_log10()
@@ -1011,6 +1021,7 @@ impl<F: Fixed> Wrapping<F> {
     /// assert_eq!(frac_1_512.recip(), Wrapping(I8F24::ZERO));
     /// ```
     #[inline]
+    #[must_use]
     pub fn recip(self) -> Wrapping<F> {
         Wrapping(self.0.wrapping_recip())
     }
@@ -1165,6 +1176,48 @@ impl<F: Fixed> Wrapping<F> {
     pub fn rem_euclid_int(self, divisor: F::Bits) -> Wrapping<F> {
         Wrapping(self.0.wrapping_rem_euclid_int(divisor))
     }
+
+    /// Linear interpolation between `start` and `end`.
+    ///
+    /// See also
+    /// <code>FixedI32::[wrapping\_lerp][FixedI32::wrapping_lerp]</code> and
+    /// <code>FixedU32::[wrapping\_lerp][FixedU32::wrapping_lerp]</code>.
+    ///
+    /// # Examples
+    ///
+    /// ```rust
+    /// use fixed::{types::I16F16, Wrapping};
+    /// type Wr = Wrapping<I16F16>;
+    /// assert_eq!(Wr::from_num(0.5).lerp(Wr::ZERO, Wr::MAX), Wr::MAX / 2);
+    /// assert_eq!(Wr::from_num(1.5).lerp(Wr::ZERO, Wr::MAX), Wr::MAX + Wr::MAX / 2);
+    /// ```
+    #[inline]
+    #[must_use]
+    pub fn lerp(self, start: Wrapping<F>, end: Wrapping<F>) -> Wrapping<F> {
+        Wrapping(self.0.wrapping_lerp(start.0, end.0))
+    }
+
+    /// Inverse linear interpolation between `start` and `end`.
+    ///
+    /// See also
+    /// <code>FixedI32::[wrapping\_inv\_lerp][FixedI32::wrapping_inv_lerp]</code> and
+    /// <code>FixedU32::[wrapping\_inv\_lerp][FixedU32::wrapping_inv_lerp]</code>.
+    ///
+    /// # Examples
+    ///
+    /// ```rust
+    /// use fixed::{types::I16F16, Wrapping};
+    /// type Wr = Wrapping<I16F16>;
+    /// assert_eq!(
+    ///     Wr::from_num(25).inv_lerp(Wr::from_num(20), Wr::from_num(40)),
+    ///     Wr::from_num(0.25)
+    /// );
+    /// ```
+    #[inline]
+    #[must_use]
+    pub fn inv_lerp(self, start: Wrapping<F>, end: Wrapping<F>) -> Wrapping<F> {
+        Wrapping(self.0.wrapping_inv_lerp(start.0, end.0))
+    }
 }
 
 impl<F: FixedSigned> Wrapping<F> {
@@ -1193,7 +1246,7 @@ impl<F: FixedSigned> Wrapping<F> {
         self.0.signed_bits()
     }
 
-    /// Returns [`true`] if the number is > 0.
+    /// Returns [`true`] if the number is >&nbsp;0.
     ///
     /// See also <code>FixedI32::[is\_positive][FixedI32::is_positive]</code>.
     ///
@@ -1210,7 +1263,7 @@ impl<F: FixedSigned> Wrapping<F> {
         self.0.is_positive()
     }
 
-    /// Returns [`true`] if the number is < 0.
+    /// Returns [`true`] if the number is <&nbsp;0.
     ///
     /// See also <code>FixedI32::[is\_negative][FixedI32::is_negative]</code>.
     ///
@@ -1243,6 +1296,7 @@ impl<F: FixedSigned> Wrapping<F> {
     /// assert_eq!(Wrapping(I16F16::MIN).abs(), Wrapping(I16F16::MIN));
     /// ```
     #[inline]
+    #[must_use]
     pub fn abs(self) -> Wrapping<F> {
         Wrapping(self.0.wrapping_abs())
     }
@@ -1251,7 +1305,7 @@ impl<F: FixedSigned> Wrapping<F> {
     ///
     /// # Warning
     ///
-    /// Using this method when 1 and −1 cannot be represented is
+    /// Using this method when 1 and &minus;1 cannot be represented is
     /// almost certainly a bug, however, this is allowed and gives the
     /// following wrapped results.
     ///
@@ -1260,9 +1314,9 @@ impl<F: FixedSigned> Wrapping<F> {
     ///     is always zero.
     ///   * When there is one integer bit, for example for the type
     ///     <code>[Wrapping]&lt;[I1F15]&gt;</code>, the return value
-    ///     is zero when `self` is zero, and −1 otherwise. This means
-    ///     that for a positive number, −1 is returned, because +1
-    ///     does not fit and is wrapped to −1.
+    ///     is zero when `self` is zero, and &minus;1 otherwise. This means
+    ///     that for a positive number, &minus;1 is returned, because +1
+    ///     does not fit and is wrapped to &minus;1.
     ///
     /// See also
     /// <code>FixedI32::[wrapping\_signum][FixedI32::wrapping_signum]</code>.
@@ -1286,6 +1340,7 @@ impl<F: FixedSigned> Wrapping<F> {
     /// [I0F16]: crate::types::I0F16
     /// [I1F15]: crate::types::I1F15
     #[inline]
+    #[must_use]
     pub fn signum(self) -> Wrapping<F> {
         Wrapping(self.0.wrapping_signum())
     }
@@ -1333,8 +1388,8 @@ impl<F: FixedUnsigned> Wrapping<F> {
     /// Returns the highest one in the binary representation, or zero
     /// if `self` is zero.
     ///
-    /// If `self` > 0, the highest one is equal to the largest power
-    /// of two that is ≤ `self`.
+    /// If `self`&nbsp;>&nbsp;0, the highest one is equal to the largest power
+    /// of two that is ≤&nbsp;`self`.
     ///
     /// See also <code>FixedU32::[highest\_one][FixedU32::highest_one]</code>.
     ///
@@ -1350,11 +1405,12 @@ impl<F: FixedUnsigned> Wrapping<F> {
     /// assert_eq!(T::ZERO.highest_one(), T::ZERO);
     /// ```
     #[inline]
+    #[must_use]
     pub fn highest_one(self) -> Wrapping<F> {
         Wrapping(self.0.highest_one())
     }
 
-    /// Returns the smallest power of two that is ≥ `self`.
+    /// Returns the smallest power of two that is ≥&nbsp;`self`.
     ///
     /// If the next power of two is too large to fit, it is wrapped to zero.
     ///
@@ -1374,6 +1430,7 @@ impl<F: FixedUnsigned> Wrapping<F> {
     /// assert_eq!(T::MAX.next_power_of_two(), T::ZERO);
     /// ```
     #[inline]
+    #[must_use]
     pub fn next_power_of_two(self) -> Wrapping<F> {
         Wrapping(self.0.wrapping_next_power_of_two())
     }
